@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2012,2014-2017                      Université de Bordeaux
- * Copyright (C) 2011-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2012,2014-2017, 2019                      Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2011-2013,2016,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -78,7 +78,7 @@ static starpu_data_handle_t A_handle, b_handle, x_handle;
 static TYPE *A, *b, *x;
 
 #ifdef STARPU_QUICK_CHECK
-static int i_max = 10;
+static int i_max = 5;
 #elif !defined(STARPU_LONG_CHECK)
 static int i_max = 100;
 #else
@@ -391,6 +391,11 @@ static void parse_args(int argc, char **argv)
 	        if (strcmp(argv[i], "-maxiter") == 0)
 		{
 			i_max = atoi(argv[++i]);
+			if (i_max <= 0)
+			{
+				FPRINTF(stderr, "the number of iterations must be positive, not %d\n", i_max);
+				exit(EXIT_FAILURE);
+			}
 			continue;
 		}
 
@@ -422,10 +427,6 @@ int main(int argc, char **argv)
 	if (starpu_get_env_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
 		return 77;
 
-#ifdef STARPU_QUICK_CHECK
-	i_max = 16;
-#endif
-
 	parse_args(argc, argv);
 
 	ret = starpu_init(NULL);
