@@ -1,8 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2013, 2015  Université de Bordeaux
- * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2008-2017                                Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2010                                     Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -50,17 +52,6 @@ static struct starpu_task *create_task(starpu_tag_t id)
  *	Create the codelets
  */
 
-static struct starpu_codelet cl11 =
-{
-	.modes = { STARPU_RW },
-	.cpu_funcs = {chol_cpu_codelet_update_u11},
-#ifdef STARPU_USE_CUDA
-	.cuda_funcs = {chol_cublas_codelet_update_u11},
-#endif
-	.nbuffers = 1,
-	.model = &chol_model_11
-};
-
 static struct starpu_task * create_task_11(starpu_data_handle_t dataA, unsigned k, unsigned reclevel)
 {
 /*	FPRINTF(stdout, "task 11 k = %d TAG = %llx\n", k, (TAG11(k))); */
@@ -87,17 +78,6 @@ static struct starpu_task * create_task_11(starpu_data_handle_t dataA, unsigned
 	return task;
 }
 
-static struct starpu_codelet cl21 =
-{
-	.modes = { STARPU_R, STARPU_RW },
-	.cpu_funcs = {chol_cpu_codelet_update_u21},
-#ifdef STARPU_USE_CUDA
-	.cuda_funcs = {chol_cublas_codelet_update_u21},
-#endif
-	.nbuffers = 2,
-	.model = &chol_model_21
-};
-
 static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned j, unsigned reclevel)
 {
 	int ret;
@@ -133,17 +113,6 @@ static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned j, un
 	return ret;
 }
 
-static struct starpu_codelet cl22 =
-{
-	.modes = { STARPU_R, STARPU_R, STARPU_RW },
-	.cpu_funcs = {chol_cpu_codelet_update_u22},
-#ifdef STARPU_USE_CUDA
-	.cuda_funcs = {chol_cublas_codelet_update_u22},
-#endif
-	.nbuffers = 3,
-	.model = &chol_model_22
-};
-
 static int create_task_22(starpu_data_handle_t dataA, unsigned k, unsigned i, unsigned j, unsigned reclevel)
 {
 	int ret;
@@ -203,7 +172,7 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 
 	/* monitor and partition the A matrix into blocks :
 	 * one block is now determined by 2 unsigned (i,j) */
-	starpu_matrix_data_register(&dataA, 0, (uintptr_t)matA, ld, size, size, sizeof(float));
+	starpu_matrix_data_register(&dataA, STARPU_MAIN_RAM, (uintptr_t)matA, ld, size, size, sizeof(float));
 
 	starpu_data_set_sequential_consistency_flag(dataA, 0);
 
@@ -223,6 +192,7 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 
 	for (k = 0; k < nbigblocks; k++)
 	{
+		starpu_iteration_push(k);
 		struct starpu_task *task = create_task_11(dataA, k, reclevel);
 		/* we defer the launch of the first task */
 		if (k == 0)
@@ -250,6 +220,7 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 				}
 			}
 		}
+		starpu_iteration_pop();
 	}
 
 	/* schedule the codelet */
@@ -264,7 +235,7 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 	{
 		/* stall the application until the end of computations */
 		starpu_tag_wait(TAG11_AUX(nblocks-1, reclevel));
-		starpu_data_unpartition(dataA, 0);
+		starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 		starpu_data_unregister(dataA);
 		return 0;
 	}
@@ -288,7 +259,7 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 
 		free(tag_array);
 
-		starpu_data_unpartition(dataA, 0);
+		starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 		starpu_data_unregister(dataA);
 
 		float *newmatA = &matA[nbigblocks*(size/nblocks)*(ld+1)];
@@ -297,9 +268,10 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 	}
 }
 
-static void initialize_system(float **A, unsigned dim, unsigned pinned)
+static void initialize_system(int argc, char **argv, float **A, unsigned pinned)
 {
 	int ret;
+	int flags = STARPU_MALLOC_SIMULATION_FOLDED;
 
 #ifdef STARPU_HAVE_MAGMA
 	magma_init();
@@ -310,19 +282,28 @@ static void initialize_system(float **A, unsigned dim, unsigned pinned)
 		exit(77);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-#ifndef STARPU_SIMGRID
-	if (pinned)
-	{
-		starpu_malloc((void **)A, dim*dim*sizeof(float));
-	}
-	else
-	{
-		*A = malloc(dim*dim*sizeof(float));
-	}
+	init_sizes();
+
+	parse_args(argc, argv);
+
+#ifdef STARPU_USE_CUDA
+	initialize_chol_model(&chol_model_11,"chol_model_11",cpu_chol_task_11_cost,cuda_chol_task_11_cost);
+	initialize_chol_model(&chol_model_21,"chol_model_21",cpu_chol_task_21_cost,cuda_chol_task_21_cost);
+	initialize_chol_model(&chol_model_22,"chol_model_22",cpu_chol_task_22_cost,cuda_chol_task_22_cost);
+#else
+	initialize_chol_model(&chol_model_11,"chol_model_11",cpu_chol_task_11_cost,NULL);
+	initialize_chol_model(&chol_model_21,"chol_model_21",cpu_chol_task_21_cost,NULL);
+	initialize_chol_model(&chol_model_22,"chol_model_22",cpu_chol_task_22_cost,NULL);
 #endif
+
+	starpu_cublas_init();
+
+	if (pinned)
+		flags |= STARPU_MALLOC_PINNED;
+	starpu_malloc_flags((void **)A, size_p*size_p*sizeof(float), flags);
 }
 
-int cholesky_grain(float *matA, unsigned size, unsigned ld, unsigned nblocks, unsigned nbigblocks, unsigned pinned)
+int cholesky_grain(float *matA, unsigned size, unsigned ld, unsigned nblocks, unsigned nbigblocks)
 {
 	double start;
 	double end;
@@ -344,17 +325,15 @@ int cholesky_grain(float *matA, unsigned size, unsigned ld, unsigned nblocks, un
 	return ret;
 }
 
-static void shutdown_system(float **matA, unsigned pinned)
+static void shutdown_system(float **matA, unsigned dim, unsigned pinned)
 {
+	int flags = STARPU_MALLOC_SIMULATION_FOLDED;
 	if (pinned)
-	{
-	     starpu_free(*matA);
-	}
-	else
-	{
-	     free(*matA);
-	}
+		flags |= STARPU_MALLOC_PINNED;
 
+	starpu_free_flags(*matA, dim*dim*sizeof(float), flags);
+
+	starpu_cublas_shutdown();
 	starpu_shutdown();
 }
 
@@ -367,19 +346,17 @@ int main(int argc, char **argv)
 
      	int ret;
 
-	parse_args(argc, argv);
-
 	float *mat = NULL;
-	initialize_system(&mat, size, pinned);
+	initialize_system(argc, argv, &mat, pinned_p);
 
 #ifndef STARPU_SIMGRID
 	unsigned i,j;
-	for (i = 0; i < size; i++)
+	for (i = 0; i < size_p; i++)
 	{
-		for (j = 0; j < size; j++)
+		for (j = 0; j < size_p; j++)
 		{
-			mat[j +i*size] = (1.0f/(1.0f+i+j)) + ((i == j)?1.0f*size:0.0f);
-			/* mat[j +i*size] = ((i == j)?1.0f*size:0.0f); */
+			mat[j +i*size_p] = (1.0f/(1.0f+i+j)) + ((i == j)?1.0f*size_p:0.0f);
+			/* mat[j +i*size_p] = ((i == j)?1.0f*size_p:0.0f); */
 		}
 	}
 #endif
@@ -388,13 +365,13 @@ int main(int argc, char **argv)
 #ifdef CHECK_OUTPUT
 	FPRINTF(stdout, "Input :\n");
 
-	for (j = 0; j < size; j++)
+	for (j = 0; j < size_p; j++)
 	{
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size_p; i++)
 		{
 			if (i <= j)
 			{
-				FPRINTF(stdout, "%2.2f\t", mat[j +i*size]);
+				FPRINTF(stdout, "%2.2f\t", mat[j +i*size_p]);
 			}
 			else
 			{
@@ -405,43 +382,43 @@ int main(int argc, char **argv)
 	}
 #endif
 
-	ret = cholesky_grain(mat, size, size, nblocks, nbigblocks, pinned);
+	ret = cholesky_grain(mat, size_p, size_p, nblocks_p, nbigblocks_p);
 
 #ifdef CHECK_OUTPUT
 	FPRINTF(stdout, "Results :\n");
 
-	for (j = 0; j < size; j++)
+	for (j = 0; j < size_p; j++)
 	{
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size_p; i++)
 		{
 			if (i <= j)
 			{
-				FPRINTF(stdout, "%2.2f\t", mat[j +i*size]);
+				FPRINTF(stdout, "%2.2f\t", mat[j +i*size_p]);
 			}
 			else
 			{
 				FPRINTF(stdout, ".\t");
-				mat[j+i*size] = 0.0f; /* debug */
+				mat[j+i*size_p] = 0.0f; /* debug */
 			}
 		}
 		FPRINTF(stdout, "\n");
 	}
 
 	FPRINTF(stderr, "compute explicit LLt ...\n");
-	float *test_mat = malloc(size*size*sizeof(float));
+	float *test_mat = malloc(size_p*size_p*sizeof(float));
 	STARPU_ASSERT(test_mat);
 
-	STARPU_SSYRK("L", "N", size, size, 1.0f,
-				mat, size, 0.0f, test_mat, size);
+	STARPU_SSYRK("L", "N", size_p, size_p, 1.0f,
+		     mat, size_p, 0.0f, test_mat, size_p);
 
 	FPRINTF(stderr, "comparing results ...\n");
-	for (j = 0; j < size; j++)
+	for (j = 0; j < size_p; j++)
 	{
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size_p; i++)
 		{
 			if (i <= j)
 			{
-                                FPRINTF(stdout, "%2.2f\t", test_mat[j +i*size]);
+                                FPRINTF(stdout, "%2.2f\t", test_mat[j +i*size_p]);
 			}
 			else
 			{
@@ -453,6 +430,6 @@ int main(int argc, char **argv)
 	free(test_mat);
 #endif
 
-	shutdown_system(&mat, pinned);
+	shutdown_system(&mat, size_p, pinned_p);
 	return ret;
 }
