@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2015,2017                           CNRS
+ * Copyright (C) 2012,2013,2017                           Inria
+ * Copyright (C) 2010-2015,2017,2019                      CNRS
  * Copyright (C) 2009-2011,2013-2015                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -51,7 +51,12 @@ int main(int argc, char **argv)
 	unsigned i;
 	unsigned niter = 50;
 
-	ret = starpu_init(NULL);
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	conf.nmic = 0;
+	conf.nmpi_ms = 0;
+
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
