@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2013,2015-2016                      CNRS
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
  * Copyright (C) 2011,2013-2016                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -48,7 +48,7 @@ void cpu_func(void *buffers[], void *cl_arg)
         }
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
 	unsigned j;
 	int n=1;
