@@ -1,8 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2011, 2014-2015  Université de Bordeaux
- * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012, 2016  CNRS
+ * Copyright (C) 2009-2011,2014-2017, 2019                      Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010                                     Mehdi Juhoor
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,11 +38,6 @@
 					| ((unsigned long long)(i)<<16)	\
 					| (unsigned long long)(j))))
 
-struct starpu_perfmodel model_11;
-struct starpu_perfmodel model_12;
-struct starpu_perfmodel model_21;
-struct starpu_perfmodel model_22;
-
 /*
  *	Construct the DAG
  */
@@ -329,7 +326,7 @@ static void dw_factoLU_grain_inner(float *matA, unsigned size, unsigned inner_si
 			tag_array[ind++] = TAG22(maxk-1, i, j, tag_prefix);
 		}
 
-		starpu_tag_wait_array(ndeps_tags, tag_array);
+		starpu_tag_wait_array(ind, tag_array);
 
 		free(tag_array);
 
