@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2012,2014-2015,2017                 Université de Bordeaux
- * Copyright (C) 2010-2013,2015-2016                      CNRS
+ * Copyright (C) 2008-2012,2014,2015,2017,2019            Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,8 +22,13 @@
 #include "heat.h"
 
 /* default values */
+#ifdef STARPU_QUICK_CHECK
+static unsigned ntheta = 8+2;
+static unsigned nthick = 8+2;
+#else
 static unsigned ntheta = 32+2;
 static unsigned nthick = 32+2;
+#endif
 static unsigned nblocks = 16;
 static unsigned nbigblocks = 8;
 static unsigned shape = 0;
@@ -326,7 +331,7 @@ static float compute_A_value(unsigned i, unsigned j, point *pmesh)
 	theta_j = NODE_TO_THETA(j);
 
 	/* Compute the Sum of all the integral over all triangles */
-	if ( (abs(thick_i - thick_j) <= 1) && (abs(theta_i - theta_j) <= 1) )
+	if ( (abs((int)thick_i - (int)thick_j) <= 1) && (abs((int)theta_i - (int)theta_j) <= 1) )
 	{
 		if ( (theta_j == theta_i -1) && (thick_j == thick_i +1))
 			goto done;
@@ -795,11 +800,13 @@ int main(int argc, char **argv)
 
 		starpu_cublas_shutdown();
 		starpu_shutdown();
-		free_system(A, B, newsize, pinned);
+		free_system(A, B, pinned);
 	}
 
 #ifdef STARPU_OPENGL_RENDER
-	opengl_render(ntheta, nthick, result, pmesh, argc, argv);
+	const char *display = getenv("DISPLAY");
+	if (display && display[0])
+		opengl_render(ntheta, nthick, result, pmesh, argc, argv);
 #endif
 
 	free(pmesh);
