@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2012, 2014-2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2015  CNRS
+ * Copyright (C) 2009-2018                                Université de Bordeaux
+ * Copyright (C) 2011,2012                                Inria
+ * Copyright (C) 2010-2012,2015,2017,2019                 CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,12 +37,12 @@ static const TYPE m1 = -1.0f;
 #endif
 
 /*
- *   U22 
+ *   U22
  */
 
-static inline void STARPU_LU(common_u22)(void *descr[],
-				int s, STARPU_ATTRIBUTE_UNUSED void *_args)
+static inline void STARPU_LU(common_u22)(void *descr[], int s, void *_args)
 {
+	(void)_args;
 	TYPE *right 	= (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
 	TYPE *left 	= (TYPE *)STARPU_MATRIX_GET_PTR(descr[1]);
 	TYPE *center 	= (TYPE *)STARPU_MATRIX_GET_PTR(descr[2]);
@@ -62,7 +63,7 @@ static inline void STARPU_LU(common_u22)(void *descr[],
 	switch (s)
 	{
 		case 0:
-			CPU_GEMM("N", "N", dy, dx, dz, 
+			CPU_GEMM("N", "N", dy, dx, dz,
 				(TYPE)-1.0, right, ld21, left, ld12,
 				(TYPE)1.0, center, ld22);
 			break;
@@ -106,6 +107,8 @@ static struct starpu_perfmodel STARPU_LU(model_22) =
 	.symbol = STARPU_LU_STR(lu_model_22_atlas)
 #elif defined(STARPU_GOTO)
 	.symbol = STARPU_LU_STR(lu_model_22_goto)
+#elif defined(STARPU_OPENBLAS)
+	.symbol = STARPU_LU_STR(lu_model_22_openblas)
 #else
 	.symbol = STARPU_LU_STR(lu_model_22)
 #endif
@@ -114,8 +117,10 @@ static struct starpu_perfmodel STARPU_LU(model_22) =
 #ifdef STARPU_USE_CUDA
 static int can_execute(unsigned workerid, struct starpu_task *task, unsigned nimpl)
 {
+	(void)task;
+	(void)nimpl;
 	enum starpu_worker_archtype type = starpu_worker_get_type(workerid);
-	if (type == STARPU_CPU_WORKER || type == STARPU_MIC_WORKER || type == STARPU_SCC_WORKER)
+	if (type == STARPU_CPU_WORKER || type == STARPU_MIC_WORKER)
 		return 1;
 
 #ifdef STARPU_SIMGRID
@@ -161,13 +166,13 @@ struct starpu_codelet cl22 =
  * U12
  */
 
-static inline void STARPU_LU(common_u12)(void *descr[],
-				int s, STARPU_ATTRIBUTE_UNUSED void *_args)
+static inline void STARPU_LU(common_u12)(void *descr[], int s, void *_args)
 {
+	(void)_args;
 	TYPE *sub11;
 	TYPE *sub12;
 
-	sub11 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);	
+	sub11 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
 	sub12 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[1]);
 
 	unsigned ld11 = STARPU_MATRIX_GET_LD(descr[0]);
@@ -225,6 +230,8 @@ static struct starpu_perfmodel STARPU_LU(model_12) =
 	.symbol = STARPU_LU_STR(lu_model_12_atlas)
 #elif defined(STARPU_GOTO)
 	.symbol = STARPU_LU_STR(lu_model_12_goto)
+#elif defined(STARPU_OPENBLAS)
+	.symbol = STARPU_LU_STR(lu_model_12_openblas)
 #else
 	.symbol = STARPU_LU_STR(lu_model_12)
 #endif
@@ -246,13 +253,13 @@ struct starpu_codelet cl12 =
 	.model = &STARPU_LU(model_12)
 };
 
-/* 
+/*
  * U21
  */
 
-static inline void STARPU_LU(common_u21)(void *descr[],
-				int s, STARPU_ATTRIBUTE_UNUSED void *_args)
+static inline void STARPU_LU(common_u21)(void *descr[], int s, void *_args)
 {
+	(void)_args;
 	TYPE *sub11;
 	TYPE *sub21;
 
@@ -264,7 +271,7 @@ static inline void STARPU_LU(common_u21)(void *descr[],
 
 	unsigned nx21 = STARPU_MATRIX_GET_NX(descr[1]);
 	unsigned ny21 = STARPU_MATRIX_GET_NY(descr[1]);
-	
+
 #ifdef STARPU_USE_CUDA
 	cublasStatus status;
 #endif
@@ -303,7 +310,7 @@ void STARPU_LU(cublas_u21)(void *descr[], void *_args)
 {
 	STARPU_LU(common_u21)(descr, 1, _args);
 }
-#endif 
+#endif
 
 static struct starpu_perfmodel STARPU_LU(model_21) =
 {
@@ -312,6 +319,8 @@ static struct starpu_perfmodel STARPU_LU(model_21) =
 	.symbol = STARPU_LU_STR(lu_model_21_atlas)
 #elif defined(STARPU_GOTO)
 	.symbol = STARPU_LU_STR(lu_model_21_goto)
+#elif defined(STARPU_OPENBLAS)
+	.symbol = STARPU_LU_STR(lu_model_21_openblas)
 #else
 	.symbol = STARPU_LU_STR(lu_model_21)
 #endif
@@ -337,12 +346,12 @@ struct starpu_codelet cl21 =
  *	U11
  */
 
-static inline void STARPU_LU(common_u11)(void *descr[],
-				int s, STARPU_ATTRIBUTE_UNUSED void *_args)
+static inline void STARPU_LU(common_u11)(void *descr[], int s, void *_args)
 {
+	(void)_args;
 	TYPE *sub11;
 
-	sub11 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]); 
+	sub11 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
 
 	unsigned long nx = STARPU_MATRIX_GET_NX(descr[0]);
 	unsigned long ld = STARPU_MATRIX_GET_LD(descr[0]);
@@ -365,7 +374,7 @@ static inline void STARPU_LU(common_u11)(void *descr[],
 				STARPU_ASSERT(!ISZERO(pivot));
 
 				CPU_SCAL(nx - z - 1, (1.0/pivot), &sub11[z+(z+1)*ld], ld);
-		
+
 				CPU_GER(nx - z - 1, nx - z - 1, -1.0,
 						&sub11[(z+1)+z*ld], 1,
 						&sub11[z+(z+1)*ld], ld,
@@ -390,7 +399,7 @@ static inline void STARPU_LU(common_u11)(void *descr[],
 						(CUBLAS_TYPE*)&inv_pivot, (CUBLAS_TYPE*)&sub11[z+(z+1)*ld], ld);
 				if (status != CUBLAS_STATUS_SUCCESS)
 					STARPU_CUBLAS_REPORT_ERROR(status);
-				
+
 				status = CUBLAS_GER(handle,
 						nx - z - 1, nx - z - 1,
 						(CUBLAS_TYPE*)&m1,
@@ -400,7 +409,7 @@ static inline void STARPU_LU(common_u11)(void *descr[],
 				if (status != CUBLAS_STATUS_SUCCESS)
 					STARPU_CUBLAS_REPORT_ERROR(status);
 			}
-			
+
 			cudaStreamSynchronize(stream);
 
 			break;
@@ -430,6 +439,8 @@ static struct starpu_perfmodel STARPU_LU(model_11) =
 	.symbol = STARPU_LU_STR(lu_model_11_atlas)
 #elif defined(STARPU_GOTO)
 	.symbol = STARPU_LU_STR(lu_model_11_goto)
+#elif defined(STARPU_OPENBLAS)
+	.symbol = STARPU_LU_STR(lu_model_11_openblas)
 #else
 	.symbol = STARPU_LU_STR(lu_model_11)
 #endif
@@ -459,7 +470,7 @@ static inline void STARPU_LU(common_u11_pivot)(void *descr[],
 {
 	TYPE *sub11;
 
-	sub11 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]); 
+	sub11 = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
 
 	unsigned long nx = STARPU_MATRIX_GET_NX(descr[0]);
 	unsigned long ld = STARPU_MATRIX_GET_LD(descr[0]);
@@ -504,7 +515,7 @@ static inline void STARPU_LU(common_u11_pivot)(void *descr[],
 				STARPU_ASSERT(!ISZERO(pivot));
 
 				CPU_SCAL(nx - z - 1, (1.0/pivot), &sub11[z+(z+1)*ld], ld);
-		
+
 				CPU_GER(nx - z - 1, nx - z - 1, -1.0,
 						&sub11[(z+1)+z*ld], 1,
 						&sub11[z+(z+1)*ld], ld,
@@ -532,7 +543,7 @@ static inline void STARPU_LU(common_u11_pivot)(void *descr[],
 					piv_ind -= 1;
 					if (status != CUBLAS_STATUS_SUCCESS)
 						STARPU_CUBLAS_REPORT_ERROR(status);
-	
+
 					ipiv[z + first] = piv_ind + z + first;
 
 					/* swap if needed */
@@ -559,7 +570,7 @@ static inline void STARPU_LU(common_u11_pivot)(void *descr[],
 						(CUBLAS_TYPE*)&sub11[z+(z+1)*ld], ld);
 				if (status != CUBLAS_STATUS_SUCCESS)
 					STARPU_CUBLAS_REPORT_ERROR(status);
-				
+
 				status = CUBLAS_GER(handle,
 						nx - z - 1, nx - z - 1,
 						(CUBLAS_TYPE*)&m1,
@@ -599,6 +610,8 @@ static struct starpu_perfmodel STARPU_LU(model_11_pivot) =
 	.symbol = STARPU_LU_STR(lu_model_11_pivot_atlas)
 #elif defined(STARPU_GOTO)
 	.symbol = STARPU_LU_STR(lu_model_11_pivot_goto)
+#elif defined(STARPU_OPENBLAS)
+	.symbol = STARPU_LU_STR(lu_model_11_pivot_openblas)
 #else
 	.symbol = STARPU_LU_STR(lu_model_11_pivot)
 #endif
@@ -628,7 +641,7 @@ static inline void STARPU_LU(common_pivot)(void *descr[],
 {
 	TYPE *matrix;
 
-	matrix = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]); 
+	matrix = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
 	unsigned long nx = STARPU_MATRIX_GET_NX(descr[0]);
 	unsigned long ld = STARPU_MATRIX_GET_LD(descr[0]);
 
@@ -700,6 +713,8 @@ static struct starpu_perfmodel STARPU_LU(model_pivot) =
 	.symbol = STARPU_LU_STR(lu_model_pivot_atlas)
 #elif defined(STARPU_GOTO)
 	.symbol = STARPU_LU_STR(lu_model_pivot_goto)
+#elif defined(STARPU_OPENBLAS)
+	.symbol = STARPU_LU_STR(lu_model_pivot_openblas)
 #else
 	.symbol = STARPU_LU_STR(lu_model_pivot)
 #endif
