@@ -1,7 +1,8 @@
 #!/bin/sh
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2012,2014-2015                           CNRS
+# Copyright (C) 2012,2014,2015,2017,2018                 CNRS
+# Copyright (C) 2017,2019                                Université de Bordeaux
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -28,7 +29,12 @@ then
     exit 77
 fi
 
-SCHEDULERS=`STARPU_SCHED="help" ./basic_examples/hello_world 2>&1 | awk '/\t->/ {print $1}'`
+if [ -n "$STARPU_SCHED" ]
+then
+	SCHEDULERS="$STARPU_SCHED"
+else
+	SCHEDULERS=`../tools/starpu_sched_display | grep -v pheft | grep -v peager | grep -v heteroprio | grep -v modular-gemm`
+fi
 
 for sched in $SCHEDULERS
 do
