@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
- * Copyright (C) 2010-2013, 2015  Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2015                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -73,7 +74,8 @@ void scal_cpu_func(void *buffers[], void *_args)
 	int begin = get_first_element_rank(n, rank, nb_workers);
 
 
-	for (i = 0; i < nel_worker; i++) {
+	for (i = 0; i < nel_worker; i++)
+	{
 		rank = i + begin;
 
 		float v = val[rank];
@@ -96,27 +98,19 @@ static struct starpu_codelet cl =
 	.type = STARPU_SPMD,
 	.max_parallelism = INT_MAX,
 	.cpu_funcs = {scal_cpu_func},
+	.cpu_funcs_name = {"scal_cpu_func"},
 	.nbuffers = 1,
 	.model = &vector_scal_model,
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	struct starpu_conf conf;
 	float *vector;
 	unsigned i;
 	int ret;
 
-	vector = malloc(NX*sizeof(*vector));
-
-	for (i = 0; i < NX; i++)
-		vector[i] = (i+1.0f);
-
-	FPRINTF(stderr, "BEFORE: First element was %f\n", vector[0]);
-	FPRINTF(stderr, "BEFORE: Last element was %f\n", vector[NX-1]);
-
 	starpu_conf_init(&conf);
-
 	conf.single_combined_worker = 1;
 	conf.sched_policy_name = "pheft";
 
@@ -124,12 +118,20 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+	vector = malloc(NX*sizeof(*vector));
+	for (i = 0; i < NX; i++)
+		vector[i] = (i+1.0f);
+
+	FPRINTF(stderr, "BEFORE: First element was %f\n", vector[0]);
+	FPRINTF(stderr, "BEFORE: Last element was %f\n", vector[NX-1]);
+
 	starpu_data_handle_t vector_handle;
-	starpu_vector_data_register(&vector_handle, 0, (uintptr_t)vector, NX, sizeof(vector[0]));
+	starpu_vector_data_register(&vector_handle, STARPU_MAIN_RAM, (uintptr_t)vector, NX, sizeof(vector[0]));
 
 	float factor = 1.001;
 
-	for (i = 0; i < ITER; i++) {
+	for (i = 0; i < ITER; i++)
+	{
 		struct starpu_task *task = starpu_task_create();
 
 		task->cl = &cl;
@@ -139,7 +141,8 @@ int main(int argc, char **argv)
 		task->cl_arg_size = sizeof(factor);
 
 		ret = starpu_task_submit(task);
-		if (ret == -ENODEV) {
+		if (ret == -ENODEV)
+		{
 		     ret = 77;
 		     break;
 		}
