@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011, 2013  Université de Bordeaux
+ * Copyright (C) 2011-2013,2016,2017                      CNRS
+ * Copyright (C) 2010,2011,2013,2014                      Université de Bordeaux
+ * Copyright (C) 2011,2012                                Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +18,7 @@
 
 #include "stencil.h"
 
-/* Perform replication of data on X and Y edges, to fold the domain on 
+/* Perform replication of data on X and Y edges, to fold the domain on
    itself through mere replication of the source state. */
 
 #define str(x) #x
@@ -75,7 +77,8 @@ opencl_shadow_init(void)
 
 void opencl_shadow_free(void)
 {
-  starpu_opencl_unload_opencl(&program);
+  int ret = starpu_opencl_unload_opencl(&program);
+  STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
 }
 
 void
@@ -88,12 +91,15 @@ opencl_shadow_host(int bz, TYPE *ptr, int nx, int ny, int nz, int ldy, int ldz,
 #endif
 
         int devid,id;
-        id = starpu_worker_get_id();
+        id = starpu_worker_get_id_check();
         devid = starpu_worker_get_devid(id);
 
         cl_kernel kernel;
         cl_command_queue cq;
-        starpu_opencl_load_kernel(&kernel, &cq, &program, "shadow", devid);
+	cl_int err;
+
+        err = starpu_opencl_load_kernel(&kernel, &cq, &program, "shadow", devid);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
         clSetKernelArg(kernel, 0, sizeof(bz), &bz);
         clSetKernelArg(kernel, 1, sizeof(ptr), &ptr);
@@ -104,12 +110,6 @@ opencl_shadow_host(int bz, TYPE *ptr, int nx, int ny, int nz, int ldy, int ldz,
         clSetKernelArg(kernel, 6, sizeof(ldz), &ldz);
         clSetKernelArg(kernel, 7, sizeof(i), &i);
 
-        cl_event ev;
-        cl_int err = clEnqueueNDRangeKernel(cq, kernel, 3, NULL, dim, NULL, 0, NULL, &ev);
-        if (err != CL_SUCCESS)
-                STARPU_OPENCL_REPORT_ERROR(err);
-        clWaitForEvents(1, &ev);
-        starpu_opencl_collect_stats(ev);
-        clReleaseEvent(ev);
+        err = clEnqueueNDRangeKernel(cq, kernel, 3, NULL, dim, NULL, 0, NULL, NULL);
+        if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 }
-
