@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
- * Copyright (C) 2010-2012, 2014, 2017  Université de Bordeaux
+ * Copyright (C) 2012,2017                                Inria
+ * Copyright (C) 2010-2013,2015-2017,2019                 CNRS
+ * Copyright (C) 2010-2012,2014,2017                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,10 +41,6 @@ static unsigned sizez = 64*SIZE;
 /* Number of blocks (scattered over the different MPI processes) */
 unsigned nbz = 64;
 
-/* StarPU top variables */
-struct starpu_top_data* starpu_top_init_loop;
-struct starpu_top_data* starpu_top_achieved_loop;
-
 /*
  *	Initialization
  */
@@ -127,12 +124,6 @@ static void init_problem(int argc, char **argv, int rank, int world_size)
 {
 	parse_args(argc, argv);
 
-	if (getenv("STARPU_TOP"))
-	{
-		starpu_top_init_loop = starpu_top_add_data_integer("Task creation iter", 0, niter, 1);
-		starpu_top_achieved_loop = starpu_top_add_data_integer("Task achieved iter", 0, niter, 1);
-		starpu_top_init_and_wait("stencil_top example");
-	}
 	create_blocks_array(sizex, sizey, sizez, nbz);
 
 	/* Select the MPI process which should compute the different blocks */
@@ -179,7 +170,7 @@ void f(unsigned task_per_worker[STARPU_NMAXWORKERS])
 	{
 		if (task_per_worker[worker])
 		{
-			char name[32];
+			char name[64];
 			starpu_worker_get_name(worker, name, sizeof(name));
 			FPRINTF(stderr,"\t%s -> %u (%2.2f%%)\n", name, task_per_worker[worker], (100.0*task_per_worker[worker])/total);
 		}
@@ -188,7 +179,7 @@ void f(unsigned task_per_worker[STARPU_NMAXWORKERS])
 
 unsigned global_workerid(unsigned local_workerid)
 {
-#ifdef STARPU_USE_MPI
+#if defined(STARPU_USE_MPI) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int rank;
 	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
 	unsigned workers_per_node = starpu_worker_get_count();
@@ -205,7 +196,7 @@ int main(int argc, char **argv)
 	int world_size;
 	int ret;
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int thread_support;
 	if (MPI_Init_thread(&argc, &argv, MPI_THREAD_SERIALIZED, &thread_support))
 	{
@@ -232,7 +223,7 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	ret = starpu_mpi_init(NULL, NULL, 0);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 #endif
@@ -246,7 +237,7 @@ int main(int argc, char **argv)
 
 	create_tasks(rank);
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int barrier_ret = MPI_Barrier(MPI_COMM_WORLD);
 	STARPU_ASSERT(barrier_ret == MPI_SUCCESS);
 #endif
@@ -263,7 +254,7 @@ int main(int argc, char **argv)
 
 	end = starpu_timing_now();
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	barrier_ret = MPI_Barrier(MPI_COMM_WORLD);
 	STARPU_ASSERT(barrier_ret == MPI_SUCCESS);
 #endif
@@ -274,7 +265,7 @@ int main(int argc, char **argv)
 
 	/*display_debug(nbz, niter, rank);*/
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	starpu_mpi_shutdown();
 #endif
 
@@ -285,7 +276,7 @@ int main(int argc, char **argv)
 	double max_timing = timing;
 	double sum_timing = timing;
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int reduce_ret;
 
 	reduce_ret = MPI_Reduce(&timing, &min_timing, 1, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
@@ -368,7 +359,7 @@ int main(int argc, char **argv)
 	free_problem(rank);
 	starpu_shutdown();
 
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	MPI_Finalize();
 #endif
 
