@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2012,2016                                Inria
  * Copyright (C) 2008-2015                                Université de Bordeaux
- * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -107,7 +107,7 @@ static void tag_cleanup_grid(unsigned piter)
 	}
 
 
-} 
+}
 
 static int create_task_grid(unsigned piter)
 {
@@ -160,9 +160,10 @@ static int create_task_grid(unsigned piter)
 }
 
 
-void callback_cpu(void *argcb STARPU_ATTRIBUTE_UNUSED)
+void callback_cpu(void *argcb)
 {
-	unsigned newcnt = STARPU_ATOMIC_ADD(&callback_cnt, -1);	
+	(void)argcb;
+	unsigned newcnt = STARPU_ATOMIC_ADD(&callback_cnt, -1);
 	ANNOTATE_HAPPENS_BEFORE(&callback_cnt);
 
 	if (newcnt == 0)
@@ -180,9 +181,10 @@ void callback_cpu(void *argcb STARPU_ATTRIBUTE_UNUSED)
 	}
 }
 
-void cpu_codelet(void *descr[] STARPU_ATTRIBUTE_UNUSED,
-			void *_args STARPU_ATTRIBUTE_UNUSED)
+void cpu_codelet(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 /*	printf("execute task\n"); */
 }
 
@@ -218,7 +220,7 @@ static void express_deps(unsigned i, unsigned j, unsigned piter)
 	}
 }
 
-int main(int argc STARPU_ATTRIBUTE_UNUSED , char **argv STARPU_ATTRIBUTE_UNUSED)
+int main(int argc, char **argv)
 {
 	int ret;
 
