@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2017  CNRS
+ * Copyright (C) 2017                                     Inria
+ * Copyright (C) 2017,2018,2019                           CNRS
+ * Copyright (C) 2017,2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,14 +18,13 @@
 
 #include <starpu_mpi.h>
 #include "helper.h"
+#include <inttypes.h>
 
-int main(int argc, char **argv)
+int main(int argc STARPU_ATTRIBUTE_UNUSED, char *argv[] STARPU_ATTRIBUTE_UNUSED)
 {
 	int flag;
-	int value;
-
-	(void) argc;
-	(void) argv;
+	int64_t *value;
+	int64_t rvalue;
 
 	starpu_mpi_comm_get_attr(MPI_COMM_WORLD, 42, NULL, &flag);
 	STARPU_ASSERT_MSG(flag == 0, "starpu_mpi_comm_get_attr was called with invalid argument\n");
@@ -31,7 +32,9 @@ int main(int argc, char **argv)
 	starpu_mpi_comm_get_attr(MPI_COMM_WORLD, STARPU_MPI_TAG_UB, &value, &flag);
 	STARPU_ASSERT_MSG(flag == 1, "starpu_mpi_comm_get_attr was called with valid argument\n");
 
-	FPRINTF(stderr, "Value: %d\n", value);
+	rvalue = *value;
+	FPRINTF(stderr, "Value: %"PRIi64"\n", *value);
+	FPRINTF(stderr, "Value: %"PRIi64"\n", rvalue);
 
 	return 0;
 }
