@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2011-2015,2017                           CNRS
- * Copyright (C) 2013-2015                                Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2011-2018                                CNRS
+ * Copyright (C) 2013-2015,2017,2018                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,36 +40,23 @@ void cpu_codelet(void *descr[], void *_args)
 	}
 }
 
-#ifdef STARPU_SIMGRID
-/* Dummy cost function for simgrid */
-static double cost_function(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, unsigned nimpl STARPU_ATTRIBUTE_UNUSED)
-{
-	return 0.000001;
-}
-static struct starpu_perfmodel dumb_model =
-{
-	.type		= STARPU_COMMON,
-	.cost_function	= cost_function
-};
-#endif
-
 static struct starpu_codelet cl =
 {
 	.cpu_funcs = {cpu_codelet},
 	.nbuffers = 1,
 	.modes = {STARPU_RW},
 #ifdef STARPU_SIMGRID
-	.model = &dumb_model,
+	.model = &starpu_perfmodel_nop,
 #endif
 };
 
-void scallback(void *arg STARPU_ATTRIBUTE_UNUSED)
+void scallback(void *arg)
 {
 	char *msg = arg;
 	FPRINTF_MPI(stderr, "Sending completed for <%s>\n", msg);
 }
 
-void rcallback(void *arg STARPU_ATTRIBUTE_UNUSED)
+void rcallback(void *arg)
 {
 	char *msg = arg;
 	FPRINTF_MPI(stderr, "Reception completed for <%s>\n", msg);
@@ -82,13 +69,19 @@ int main(int argc, char **argv)
 	starpu_data_handle_t *data_handles;
 	int size=10;
 
-	ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_init(&argc, &argv, 1);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
+	ret = starpu_mpi_init_conf(&argc, &argv, 1, MPI_COMM_WORLD, NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &nodes);
 
+	if (starpu_cpu_worker_get_count() == 0)
+	{
+		if (rank == 0)
+			FPRINTF(stderr, "We need at least 1 CPU worker.\n");
+		starpu_mpi_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
+
 	if (rank == 0)
 	{
 		/* Allocate the vector */
@@ -116,7 +109,7 @@ int main(int argc, char **argv)
 		}
 		else if (mpi_rank == rank)
 		{
-			/* I do not own that index but i will need it for my computations */
+			/* I do not own this index but i will need it for my computations */
 			starpu_vector_data_register(&data_handles[x], -1, (uintptr_t)NULL, 1, sizeof(int));
 		}
 		else
@@ -202,6 +195,5 @@ int main(int argc, char **argv)
 	free(data_handles);
 
 	starpu_mpi_shutdown();
-	starpu_shutdown();
 	return (rank == 0) ? ret : 0;
 }
