@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010,2011,2018                           Université de Bordeaux
+ * Copyright (C) 2011                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,12 +18,13 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
 soclCreateKernelsInProgram(cl_program   UNUSED(program),
-                         cl_uint        UNUSED(num_kernels),
-                         cl_kernel *    UNUSED(kernels),
-                         cl_uint *      UNUSED(num_kernels_ret)) CL_API_SUFFIX__VERSION_1_0
+			   cl_uint        UNUSED(num_kernels),
+			   cl_kernel *    UNUSED(kernels),
+			   cl_uint *      UNUSED(num_kernels_ret))
 {
-   //TODO
-   return CL_INVALID_OPERATION;
+	//TODO
+	return CL_INVALID_OPERATION;
 }
