@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2012,2015,2017                           CNRS
- * Copyright (C) 2010-2011, 2018                                Université de Bordeaux
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010-2012,2018                           Université de Bordeaux
+ * Copyright (C) 2011,2012                                Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +29,8 @@ soclEnqueueBarrier(cl_command_queue cq)
 	return CL_SUCCESS;
 }
 
-cl_int command_barrier_submit(command_barrier cmd) {
+cl_int command_barrier_submit(command_barrier cmd)
+{
 	struct starpu_task *task;
 	task = task_create(CL_COMMAND_BARRIER);
 
