@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010,2011,2018                           Université de Bordeaux
  * Copyright (C) 2011                                     Inria
- * Copyright (C) 2012,2015,2017                           CNRS
- * Copyright (C) 2010-2011, 2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,17 +27,18 @@ soclGetCommandQueueInfo(cl_command_queue    cq,
 			void *                param_value,
 			size_t *              param_value_size_ret)
 {
-   if (cq == NULL)
-      return CL_INVALID_COMMAND_QUEUE;
+	if (cq == NULL)
+		return CL_INVALID_COMMAND_QUEUE;
 
-   switch (param_name) {
-      INFO_CASE(CL_QUEUE_CONTEXT, cq->context);
-      INFO_CASE(CL_QUEUE_DEVICE, cq->device);
-      INFO_CASE(CL_QUEUE_REFERENCE_COUNT, cq->_entity.refs);
-      INFO_CASE(CL_QUEUE_PROPERTIES, cq->properties);
-      default:
-         return CL_INVALID_VALUE;
-   }
+	switch (param_name)
+	{
+		INFO_CASE(CL_QUEUE_CONTEXT, cq->context);
+		INFO_CASE(CL_QUEUE_DEVICE, cq->device);
+		INFO_CASE(CL_QUEUE_REFERENCE_COUNT, cq->_entity.refs);
+		INFO_CASE(CL_QUEUE_PROPERTIES, cq->properties);
+	default:
+		return CL_INVALID_VALUE;
+	}
 
-   return CL_SUCCESS; 
+	return CL_SUCCESS;
 }
