@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010,2011,2018                           Université de Bordeaux
+ * Copyright (C) 2011                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,12 +19,13 @@
 #include "socl.h"
 #include "getinfo.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
 soclGetImageInfo(cl_mem           UNUSED(image),
-               cl_image_info    UNUSED(param_name), 
-               size_t           UNUSED(param_value_size),
-               void *           UNUSED(param_value),
-               size_t *         UNUSED(param_value_size_ret)) CL_API_SUFFIX__VERSION_1_0
+		 cl_image_info    UNUSED(param_name),
+		 size_t           UNUSED(param_value_size),
+		 void *           UNUSED(param_value),
+		 size_t *         UNUSED(param_value_size_ret))
 {
-   return CL_INVALID_OPERATION;
+	return CL_INVALID_OPERATION;
 }
