@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010,2011,2018                           Université de Bordeaux
+ * Copyright (C) 2011                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,13 +18,14 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
-soclReleaseProgram(cl_program program) CL_API_SUFFIX__VERSION_1_0
+soclReleaseProgram(cl_program program)
 {
-  if (program == NULL)
-    return CL_INVALID_PROGRAM;
+	if (program == NULL)
+		return CL_INVALID_PROGRAM;
 
-  gc_entity_release(program);
+	gc_entity_release(program);
 
-  return CL_SUCCESS;
+	return CL_SUCCESS;
 }
