@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010,2011                                Université de Bordeaux
  * Copyright (C) 2011                                     Inria
- * Copyright (C) 2012,2015,2017                           CNRS
- * Copyright (C) 2010-2011                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,8 @@
 
 typedef struct command_list_t * command_list;
 
-struct command_list_t {
+struct command_list_t
+{
 	cl_command cmd;
 	command_list next;
 	command_list prev;
