@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2012,2014-2015,2017                      CNRS
+ * Copyright (C) 2011,2012                                Inria
+ * Copyright (C) 2012,2014,2015,2017,2018                 CNRS
  * Copyright (C) 2010-2013                                Université de Bordeaux
  * Copyright (C) 2012                                     Vincent Danjean
  *
@@ -27,12 +27,11 @@
  * its command queue.
  */
 
-
-void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_events, const cl_event * events) {
-
-  cl_event ev = command_event_get_ex(cmd);
-  ev->prof_queued = _socl_nanotime();
-  gc_entity_release(ev);
+void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_events, const cl_event * events)
+{
+	cl_event ev = command_event_get_ex(cmd);
+	ev->prof_queued = _socl_nanotime();
+	gc_entity_release(ev);
 
 	/* Check if the command is a barrier */
 	int is_barrier = (cmd->typ == CL_COMMAND_BARRIER || !(cq->properties & CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE));
@@ -41,7 +40,7 @@ void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_e
 	gc_entity_store(&cmd->event->cq, cq);
 
 	/* Lock command queue */
-	starpu_pthread_mutex_lock(&cq->mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&cq->mutex);
 
 	/*** Number of dependencies ***/
 	int ndeps = num_events;
@@ -51,9 +50,11 @@ void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_e
 		ndeps++;
 
 	/* Add dependencies to out-of-order events (if any) */
-	if (is_barrier) {
+	if (is_barrier)
+	{
 		command_list cl = cq->commands;
-		while (cl != NULL) {
+		while (cl != NULL)
+		{
 			ndeps++;
 			cl = cl->next;
 		}
@@ -65,13 +66,15 @@ void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_e
 	int n = 0;
 
 	/* Add dependency to last barrier if applicable */
-	if (cq->barrier != NULL) 
-      gc_entity_store(&deps[n++], cq->barrier->event);
+	if (cq->barrier != NULL)
+		gc_entity_store(&deps[n++], cq->barrier->event);
 
 	/* Add dependencies to out-of-order events (if any) */
-	if (is_barrier) {
+	if (is_barrier)
+	{
 		command_list cl = cq->commands;
-		while (cl != NULL) {
+		while (cl != NULL)
+		{
 			gc_entity_store(&deps[n++], cl->cmd->event);
 			cl = cl->next;
 		}
@@ -79,7 +82,8 @@ void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_e
 
 	/* Add explicit dependencies */
 	unsigned i;
-	for (i=0; i<num_events; i++) {
+	for (i=0; i<num_events; i++)
+	{
 		gc_entity_store(&deps[n++], events[i]);
 	}
 
@@ -88,25 +92,27 @@ void command_queue_enqueue_ex(cl_command_queue cq, cl_command cmd, cl_uint num_e
 	cmd->events = deps;
 
 	/* Insert command in the queue */
-	if (is_barrier) {
+	if (is_barrier)
+	{
 		/* Remove out-of-order commands */
 		cq->commands = NULL;
 		/* Register the command as the last barrier */
 		cq->barrier = cmd;
 	}
-	else {
+	else
+	{
 		/* Add command to the list of out-of-order commands */
 		cq->commands = command_list_cons(cmd, cq->commands);
 	}
 
-	/* Submit command 
+	/* Submit command
 	 * We need to do it before unlocking because we don't want events to get
 	 * released while we use them to set dependencies
 	 */
 	command_submit_ex(cmd);
 
 	/* Unlock command queue */
-	starpu_pthread_mutex_unlock(&cq->mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&cq->mutex);
 
 	gc_entity_release(cmd);
 }
