@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012 University of Bordeaux
- * Copyright (C) 2012 CNRS
+ * Copyright (C) 2012,2014,2015,2017                      CNRS
+ * Copyright (C) 2010-2013,2016                           Université de Bordeaux
+ * Copyright (C) 2012                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +16,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <pthread.h>
 #include "socl.h"
 #include "gc.h"
 #include "mem_objects.h"
@@ -29,7 +29,7 @@ extern volatile int _starpu_init;
  * Initialize StarPU
  */
 
-void socl_init_starpu(void);
+int socl_init_starpu(void);
 void soclShutdown(void);
 
 #endif /* SOCL_INIT_H */
