@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2012                                     Inria
- * Copyright (C) 2010-2012,2014-2016                      Université de Bordeaux
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2012,2014-2018                      Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2018                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +29,10 @@ void _starpu_cg_list_init(struct _starpu_cg_list *list)
 	_starpu_spin_init(&list->lock);
 	list->ndeps = 0;
 	list->ndeps_completed = 0;
+#ifdef STARPU_DEBUG
+	list->deps = NULL;
+	list->done = NULL;
+#endif
 
 	list->terminated = 0;
 
@@ -51,11 +55,21 @@ void _starpu_cg_list_deinit(struct _starpu_cg_list *list)
 		 * if there is no more reference. */
 		unsigned ntags = STARPU_ATOMIC_ADD(&cg->ntags, -1);
 		if (ntags == 0)
+		{
+#ifdef STARPU_DEBUG
+			free(list->succ[id]->deps);
+			free(list->succ[id]->done);
+#endif
 			free(list->succ[id]);
+		}
 	}
 
 #ifdef STARPU_DYNAMIC_DEPS_SIZE
 	free(list->succ);
+#endif
+#ifdef STARPU_DEBUG
+	free(list->deps);
+	free(list->done);
 #endif
 	_starpu_spin_destroy(&list->lock);
 }
@@ -126,8 +140,7 @@ int _starpu_list_task_scheduled_successors_in_cg_list(struct _starpu_cg_list *su
 		if (n < ndeps)
 		{
 			struct starpu_task *task = cg->succ.job->task;
-			if (task->cl == NULL || task->cl->where == STARPU_NOWHERE
-					|| task->execute_on_a_specific_worker)
+			if (task->cl == NULL || task->where == STARPU_NOWHERE || task->execute_on_a_specific_worker)
 				/* will not be scheduled */
 				continue;
 			task_array[n] = task;
@@ -159,7 +172,7 @@ int _starpu_list_tag_successors_in_cg_list(struct _starpu_cg_list *successors, u
 }
 
 /* Note: in case of a tag, it must be already locked */
-void _starpu_notify_cg(struct _starpu_cg *cg)
+void _starpu_notify_cg(void *pred STARPU_ATTRIBUTE_UNUSED, struct _starpu_cg *cg)
 {
 	STARPU_ASSERT(cg);
 	unsigned remaining = STARPU_ATOMIC_ADD(&cg->remaining, -1);
@@ -168,6 +181,13 @@ void _starpu_notify_cg(struct _starpu_cg *cg)
 	if (remaining == 0)
 	{
 		ANNOTATE_HAPPENS_AFTER(&cg->remaining);
+		/* Note: This looks racy to helgrind when the tasks are not
+		 * autoregenerated, since they then unsubcribe from the
+		 * completion group in parallel, thus decreasing ntags. This is
+		 * however not a problem since it means we will not reuse this
+		 * cg, and remaining will not be used, so a bogus value won't
+		 * hurt.
+		 */
 		cg->remaining = cg->ntags;
 
 		/* the group is now completed */
@@ -216,6 +236,29 @@ void _starpu_notify_cg(struct _starpu_cg *cg)
 				STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 
 				job_successors = &j->job_successors;
+#ifdef STARPU_DEBUG
+				if (!j->task->regenerate)
+				{
+					unsigned i;
+					/* Remove backward cg pointers for easier debugging */
+					if (job_successors->deps)
+					{
+						for (i = 0; i < job_successors->ndeps; i++)
+							if (job_successors->deps[i] == cg)
+								break;
+						STARPU_ASSERT(i < job_successors->ndeps);
+						job_successors->done[i] = 1;
+					}
+					if (cg->deps)
+					{
+						for (i = 0; i < cg->ndeps; i++)
+							if (cg->deps[i] == pred)
+								break;
+						STARPU_ASSERT(i < cg->ndeps);
+						cg->done[i] = 1;
+					}
+				}
+#endif
 
 				unsigned ndeps_completed =
 					STARPU_ATOMIC_ADD(&job_successors->ndeps_completed, 1);
@@ -245,11 +288,69 @@ void _starpu_notify_cg(struct _starpu_cg *cg)
 	}
 }
 
+/* Note: in case of a tag, it must be already locked */
+void _starpu_notify_job_ready_soon_cg(void *pred STARPU_ATTRIBUTE_UNUSED, struct _starpu_cg *cg, _starpu_notify_job_start_data *data)
+{
+	STARPU_ASSERT(cg);
+
+	if (cg->remaining == 1)
+	{
+		/* the group is to be completed */
+		switch (cg->cg_type)
+		{
+			case STARPU_CG_APPS:
+				/* Not a task */
+				break;
+
+			case STARPU_CG_TAG:
+			{
+				struct _starpu_cg_list *tag_successors;
+				struct _starpu_tag *tag;
+
+				tag = cg->succ.tag;
+				tag_successors = &tag->tag_successors;
+
+				/* Note: the tag is already locked by the
+				 * caller. */
+				if ((tag->state == STARPU_BLOCKED) &&
+					(tag_successors->ndeps == tag_successors->ndeps_completed + 1))
+				{
+					/* This is to be ready */
+					_starpu_enforce_deps_notify_job_ready_soon(tag->job, data, 1);
+				}
+				break;
+			}
+
+ 		        case STARPU_CG_TASK:
+			{
+				struct _starpu_cg_list *job_successors;
+				struct _starpu_job *j;
+
+				j = cg->succ.job;
+				job_successors = &j->job_successors;
+
+				if (job_successors->ndeps == job_successors->ndeps_completed + 1 &&
+					j->task->status == STARPU_TASK_BLOCKED_ON_TASK)
+				{
+					/* This is to be ready */
+					_starpu_enforce_deps_notify_job_ready_soon(j, data, 0);
+				}
+
+				break;
+			}
+
+			default:
+				STARPU_ABORT();
+		}
+	}
+}
+
+
 /* Caller just has to promise that the list will not disappear.
  * _starpu_notify_cg_list protects the list itself.
  * No job lock should be held, since we might want to immediately call the callback of an empty task.
  */
-void _starpu_notify_cg_list(struct _starpu_cg_list *successors)
+void _starpu_notify_cg_list(void *pred, struct _starpu_cg_list *successors)
 {
 	unsigned succ;
 
@@ -279,7 +380,7 @@ void _starpu_notify_cg_list(struct _starpu_cg_list *successors)
 			_starpu_spin_lock(&cgtag->lock);
 		}
 
-		_starpu_notify_cg(cg);
+		_starpu_notify_cg(pred, cg);
 
 		if (cg_type == STARPU_CG_TAG)
 			_starpu_spin_unlock(&cgtag->lock);
@@ -289,3 +390,39 @@ void _starpu_notify_cg_list(struct _starpu_cg_list *successors)
 	successors->terminated = 1;
 	_starpu_spin_unlock(&successors->lock);
 }
+
+/* Caller just has to promise that the list will not disappear.
+ * _starpu_notify_cg_list protects the list itself.
+ * No job lock should be held, since we might want to immediately call the callback of an empty task.
+ */
+void _starpu_notify_job_start_cg_list(void *pred, struct _starpu_cg_list *successors, _starpu_notify_job_start_data *data)
+{
+	unsigned succ;
+
+	_starpu_spin_lock(&successors->lock);
+	/* Note: some thread might be concurrently adding other items */
+	for (succ = 0; succ < successors->nsuccs; succ++)
+	{
+		struct _starpu_cg *cg = successors->succ[succ];
+		_starpu_spin_unlock(&successors->lock);
+		STARPU_ASSERT(cg);
+		unsigned cg_type = cg->cg_type;
+
+		struct _starpu_tag *cgtag = NULL;
+
+		if (cg_type == STARPU_CG_TAG)
+		{
+			cgtag = cg->succ.tag;
+			STARPU_ASSERT(cgtag);
+			_starpu_spin_lock(&cgtag->lock);
+		}
+
+		_starpu_notify_job_ready_soon_cg(pred, cg, data);
+
+		if (cg_type == STARPU_CG_TAG)
+			_starpu_spin_unlock(&cgtag->lock);
+
+		_starpu_spin_lock(&successors->lock);
+	}
+	_starpu_spin_unlock(&successors->lock);
+}
