@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2016                                Université de Bordeaux
- * Copyright (C) 2013,2015                                Inria
- * Copyright (C) 2010-2013,2017                           CNRS
+ * Copyright (C) 2013,2015-2017                           Inria
+ * Copyright (C) 2010-2013,2016,2017                      CNRS
+ * Copyright (C) 2009-2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,8 +26,6 @@
 
 /* TODO factorize with data_concurrency.c and btw support redux */
 
-/* TODO: fine-grain R/W access */
-
 //#define LOCK_OR_DELEGATE
 
 /*
@@ -45,6 +43,7 @@
  * - for each handle h of T:
  *   - mutex_lock(&arbiter)
  *   - release reference on h
+ *   - call _starpu_notify_arbitered_dependencies which does the following
  *   - for each task Tc waiting for h:
  *     - for each data Tc_h it is waiting for:
  *       - if Tc_h is busy, goto fail
@@ -65,7 +64,7 @@
  *   - mutex_unlock(&arbiter)
  *
  *
- * at submission of task T:
+ * at submission of task T (_starpu_submit_job_enforce_arbitered_deps):
  *
  * - mutex_lock(&arbiter)
  * - for each handle h of T:
@@ -86,8 +85,26 @@
  *   - unlock(h)
  * - mutex_unlock(&arbiter)
  * - return 1;
+ *
+ * at acquire (_starpu_attempt_to_submit_arbitered_data_request):
+ * - mutex_lock(&arbiter)
+ * - try to take a reference on h
+ *   - on failure, record as waiting on h
+ * - mutex_unlock(&arbiter);
+ * - return 0 if succeeded, 1 if failed;
  */
 
+static int _starpu_arbiter_filter_modes(int mode)
+{
+	/* Do not care about some flags */
+	mode &= ~STARPU_COMMUTE;
+	mode &= ~STARPU_SSEND;
+	mode &= ~STARPU_LOCALITY;
+	if (mode == STARPU_RW)
+		mode = STARPU_W;
+	return mode;
+}
+
 struct starpu_arbiter
 {
 #ifdef LOCK_OR_DELEGATE
@@ -128,8 +145,7 @@ struct LockOrDelegateListNode
  */
 static int _starpu_LockOrDelegatePostOrPerform(starpu_arbiter_t arbiter, void (*func)(void*), void* data)
 {
-	struct LockOrDelegateListNode *newNode;
-	struct LockOrDelegateListNode *iter, *next;
+	struct LockOrDelegateListNode *newNode, *iter, *next;
 	int did = 0;
 
 	_STARPU_MALLOC(newNode, sizeof(*newNode));
@@ -188,7 +204,7 @@ static int _starpu_LockOrDelegatePostOrPerform(starpu_arbiter_t arbiter, void (*
 
 #endif
 
-/* Try to submit a data request, in case the request can be processed
+/* Try to submit just one data request, in case the request can be processed
  * immediatly, return 0, if there is still a dependency that is not compatible
  * with the current mode, the request is put in the per-handle list of
  * "requesters", and this function returns 1. */
@@ -229,7 +245,7 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 						       void (*callback)(void *), void *argcb,
 						       struct _starpu_job *j, unsigned buffer_index)
 {
-	struct starpu_submit_arbitered_args *args;
+	struct starpu_submit_arbitered_args* args;
 	_STARPU_MALLOC(args, sizeof(*args));
 	args->request_from_codelet = request_from_codelet;
 	args->handle = handle;
@@ -259,8 +275,7 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 	STARPU_PTHREAD_MUTEX_LOCK(&arbiter->mutex);
 #endif // LOCK_OR_DELEGATE
 
-	if (mode == STARPU_RW)
-		mode = STARPU_W;
+	mode = _starpu_arbiter_filter_modes(mode);
 
 	STARPU_ASSERT_MSG(!(mode & STARPU_REDUX), "REDUX with arbiter is not implemented\n");
 
@@ -273,7 +288,7 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 		while (cpt < STARPU_SPIN_MAXTRY && _starpu_spin_trylock(&handle->header_lock))
 		{
 			cpt++;
-			_starpu_datawizard_progress(_starpu_memory_node_get_local_key(), 0);
+			_starpu_datawizard_progress(0);
 		}
 		if (cpt == STARPU_SPIN_MAXTRY)
 			_starpu_spin_lock(&handle->header_lock);
@@ -286,9 +301,21 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 	/* If there is currently nobody accessing the piece of data, or it's
 	 * not another writter and if this is the same type of access as the
 	 * current one, we can proceed. */
-	unsigned put_in_list;
+	unsigned put_in_list = 1;
 
-	if (handle->refcnt)
+	if ((handle->refcnt == 0) || (!(mode == STARPU_W) && (handle->current_mode == mode)))
+	{
+		/* TODO: Detect whether this is the end of a reduction phase etc. like in data_concurrency.c */
+		if (0)
+		{
+		}
+		else
+		{
+			put_in_list = 0;
+		}
+	}
+
+	if (put_in_list)
 	{
 		/* there cannot be multiple writers or a new writer
 		 * while the data is in read mode */
@@ -300,10 +327,11 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 		r->is_requested_by_codelet = request_from_codelet;
 		r->j = j;
 		r->buffer_index = buffer_index;
+		r->prio = j ? j->task->priority : 0;
 		r->ready_data_callback = callback;
 		r->argcb = argcb;
 
-		_starpu_data_requester_list_push_back(&handle->arbitered_req_list, r);
+		_starpu_data_requester_prio_list_push_back(&handle->arbitered_req_list, r);
 
 		/* failed */
 		put_in_list = 1;
@@ -365,7 +393,7 @@ static void __starpu_submit_job_enforce_arbitered_deps(void* inData)
 
 void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned buf, unsigned nbuffers)
 {
-	struct starpu_enforce_arbitered_args *args;
+	struct starpu_enforce_arbitered_args* args;
 	_STARPU_MALLOC(args, sizeof(*args));
 	starpu_data_handle_t handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buf);
 	args->j = j;
@@ -381,7 +409,8 @@ static void ___starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, u
 #else // LOCK_OR_DELEGATE
 void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned buf, unsigned nbuffers)
 {
-	starpu_arbiter_t arbiter = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buf)->arbiter;
+	struct _starpu_data_descr *descrs = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
+	starpu_arbiter_t arbiter = descrs[buf].handle->arbiter;
 	STARPU_PTHREAD_MUTEX_LOCK(&arbiter->mutex);
 #endif
 	STARPU_ASSERT(arbiter);
@@ -395,10 +424,14 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 
 	for (idx_buf_arbiter = start_buf_arbiter; idx_buf_arbiter < nbuffers; idx_buf_arbiter++)
 	{
-		handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_arbiter);
-		mode = _STARPU_JOB_GET_ORDERED_BUFFER_MODE(j, idx_buf_arbiter);
+		handle = descrs[idx_buf_arbiter].handle;
+		mode = descrs[idx_buf_arbiter].mode & ~STARPU_COMMUTE;
 
-		if (idx_buf_arbiter && (_STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_arbiter-1)==handle))
+		mode = _starpu_arbiter_filter_modes(mode);
+
+		STARPU_ASSERT_MSG(!(mode & STARPU_REDUX), "REDUX with arbiter is not implemented\n");
+
+		if (idx_buf_arbiter && (descrs[idx_buf_arbiter-1].handle == handle))
 			/* We have already requested this data, skip it. This
 			 * depends on ordering putting writes before reads, see
 			 * _starpu_compar_handles.  */
@@ -412,12 +445,13 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 
 		/* Try to take handle */
 		_starpu_spin_lock(&handle->header_lock);
-		if (handle->refcnt == 0)
+		if ((handle->refcnt == 0) || (!(mode == STARPU_W) && (handle->current_mode == mode)))
 		{
 			/* Got it */
 			handle->refcnt++;
 			handle->busy_count++;
-			handle->current_mode = mode;
+			if (mode != STARPU_R || handle->current_mode != mode)
+				handle->current_mode = mode;
 			_starpu_spin_unlock(&handle->header_lock);
 		}
 		else
@@ -437,11 +471,12 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 		r->is_requested_by_codelet = 1;
 		r->j = j;
 		r->buffer_index = start_buf_arbiter;
+		r->prio = j->task->priority;
 		r->ready_data_callback = NULL;
 		r->argcb = NULL;
 
 		/* store node in list */
-		_starpu_data_requester_list_push_front(&handle->arbitered_req_list, r);
+		_starpu_data_requester_prio_list_push_front(&handle->arbitered_req_list, r);
 
 		_starpu_spin_lock(&handle->header_lock);
 		handle->busy_count++;
@@ -451,9 +486,9 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 		unsigned idx_buf_cancel;
 		for (idx_buf_cancel = start_buf_arbiter; idx_buf_cancel < idx_buf_arbiter ; idx_buf_cancel++)
 		{
-			starpu_data_handle_t cancel_handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_cancel);
+			starpu_data_handle_t cancel_handle = descrs[idx_buf_cancel].handle;
 
-			if (idx_buf_cancel && (_STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_cancel-1)==cancel_handle))
+			if (idx_buf_cancel && (descrs[idx_buf_cancel-1].handle == cancel_handle))
 				continue;
 			if (cancel_handle->arbiter != arbiter)
 				/* Will have to process another arbiter, will do that later */
@@ -461,7 +496,7 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 
 			_starpu_spin_lock(&cancel_handle->header_lock);
 			/* reset the counter because finally we do not take the data */
-			STARPU_ASSERT(cancel_handle->refcnt == 1);
+			STARPU_ASSERT(cancel_handle->refcnt >= 1);
 			cancel_handle->refcnt--;
 			STARPU_ASSERT(cancel_handle->busy_count > 0);
 			cancel_handle->busy_count--;
@@ -509,43 +544,75 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 #endif
 
 	/* Since the request has been posted the handle may have been proceed and released */
-	if (_starpu_data_requester_list_empty(&handle->arbitered_req_list))
+	if (_starpu_data_requester_prio_list_empty(&handle->arbitered_req_list))
 	{
+		/* No waiter, just remove our reference */
+		_starpu_spin_lock(&handle->header_lock);
+		STARPU_ASSERT(handle->refcnt > 0);
+		handle->refcnt--;
+		STARPU_ASSERT(handle->busy_count > 0);
+		handle->busy_count--;
 #ifndef LOCK_OR_DELEGATE
 		STARPU_PTHREAD_MUTEX_UNLOCK(&arbiter->mutex);
 #endif
+		if (_starpu_data_check_not_busy(handle))
+			/* Handle was even destroyed, don't unlock it.  */
+			return;
+		_starpu_spin_unlock(&handle->header_lock);
 		return;
 	}
 
+	/* There is a waiter, remove our reference */
+	_starpu_spin_lock(&handle->header_lock);
+	STARPU_ASSERT(handle->refcnt > 0);
+	handle->refcnt--;
+	STARPU_ASSERT(handle->busy_count > 0);
+	handle->busy_count--;
+	/* There should be at least one busy_count reference for the waiter
+	 * (thus we don't risk to see the handle disappear below) */
+	STARPU_ASSERT(handle->busy_count > 0);
+	_starpu_spin_unlock(&handle->header_lock);
+
 	/* Note: we may be putting back our own requests, so avoid looping by
 	 * extracting the list */
-	struct _starpu_data_requester_list l = handle->arbitered_req_list;
-	_starpu_data_requester_list_init(&handle->arbitered_req_list);
+	struct _starpu_data_requester_prio_list l = handle->arbitered_req_list;
+	_starpu_data_requester_prio_list_init(&handle->arbitered_req_list);
 
-	while (!_starpu_data_requester_list_empty(&l))
+	while (!_starpu_data_requester_prio_list_empty(&l))
 	{
-		struct _starpu_data_requester *r = _starpu_data_requester_list_pop_front(&l);
+		struct _starpu_data_requester *r = _starpu_data_requester_prio_list_pop_front_highest(&l);
 
 		if (!r->is_requested_by_codelet)
 		{
 			/* data_acquire_cb, process it */
 			enum starpu_data_access_mode r_mode = r->mode;
-			if (r_mode == STARPU_RW)
-				r_mode = STARPU_W;
+			int put_in_list = 1;
+
+			r_mode = _starpu_arbiter_filter_modes(r_mode);
 
 			_starpu_spin_lock(&handle->header_lock);
-			handle->refcnt++;
 			handle->busy_count++;
-			handle->current_mode = r_mode;
+			if ((handle->refcnt == 0) || (!(r_mode == STARPU_W) && (handle->current_mode == r_mode)))
+			{
+				handle->refcnt++;
+				handle->current_mode = r_mode;
+				put_in_list = 0;
+			}
 			_starpu_spin_unlock(&handle->header_lock);
 
+			if (put_in_list)
+				_starpu_data_requester_prio_list_push_front(&l, r);
+
 			/* Put back remaining requests */
-			_starpu_data_requester_list_push_list_back(&handle->arbitered_req_list, &l);
+			_starpu_data_requester_prio_list_push_prio_list_back(&handle->arbitered_req_list, &l);
 #ifndef LOCK_OR_DELEGATE
 			STARPU_PTHREAD_MUTEX_UNLOCK(&arbiter->mutex);
 #endif
-			r->ready_data_callback(r->argcb);
-			_starpu_data_requester_delete(r);
+			if (!put_in_list)
+			{
+				r->ready_data_callback(r->argcb);
+				_starpu_data_requester_delete(r);
+			}
 
 			_starpu_spin_lock(&handle->header_lock);
 			STARPU_ASSERT(handle->busy_count > 0);
@@ -566,11 +633,12 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 		enum starpu_data_access_mode mode;
 
 		unsigned start_buf_arbiter = r->buffer_index;
+		struct _starpu_data_descr *descrs = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
 
 		for (idx_buf_arbiter = start_buf_arbiter; idx_buf_arbiter < nbuffers; idx_buf_arbiter++)
 		{
-			handle_arbiter = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_arbiter);
-			if (idx_buf_arbiter && (_STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_arbiter-1)==handle_arbiter))
+			handle_arbiter = descrs[idx_buf_arbiter].handle;
+			if (idx_buf_arbiter && (descrs[idx_buf_arbiter-1].handle == handle_arbiter))
 				/* We have already requested this data, skip it. This
 				 * depends on ordering putting writes before reads, see
 				 * _starpu_compar_handles.  */
@@ -579,11 +647,12 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 				/* Will have to process another arbiter, will do that later */
 				break;
 
-			mode = _STARPU_JOB_GET_ORDERED_BUFFER_MODE(j, idx_buf_arbiter);
+			mode = descrs[idx_buf_arbiter].mode;
+			mode = _starpu_arbiter_filter_modes(mode);
 
 			/* we post all arbiter  */
 			_starpu_spin_lock(&handle_arbiter->header_lock);
-			if (handle_arbiter->refcnt != 0)
+			if (!((handle_arbiter->refcnt == 0) || (!(mode == STARPU_W) && (handle_arbiter->current_mode == mode))))
 			{
 				/* handle is not available, record ourself */
 				_starpu_spin_unlock(&handle_arbiter->header_lock);
@@ -609,7 +678,7 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 				_starpu_spin_unlock(&handle->header_lock);
 
 			/* Put back remaining requests */
-			_starpu_data_requester_list_push_list_back(&handle->arbitered_req_list, &l);
+			_starpu_data_requester_prio_list_push_prio_list_back(&handle->arbitered_req_list, &l);
 #ifndef LOCK_OR_DELEGATE
 			STARPU_PTHREAD_MUTEX_UNLOCK(&arbiter->mutex);
 #endif
@@ -629,7 +698,7 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 
 			/* store node in list */
 			r->mode = mode;
-			_starpu_data_requester_list_push_front(&handle_arbiter->arbitered_req_list, r);
+			_starpu_data_requester_prio_list_push_front(&handle_arbiter->arbitered_req_list, r);
 
 			/* Move check_busy reference too */
 			_starpu_spin_lock(&handle->header_lock);
@@ -646,13 +715,13 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 			unsigned idx_buf_cancel;
 			for (idx_buf_cancel = start_buf_arbiter; idx_buf_cancel < idx_buf_arbiter ; idx_buf_cancel++)
 			{
-				starpu_data_handle_t cancel_handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_cancel);
-				if (idx_buf_cancel && (_STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, idx_buf_cancel-1)==cancel_handle))
+				starpu_data_handle_t cancel_handle = descrs[idx_buf_cancel].handle;
+				if (idx_buf_cancel && (descrs[idx_buf_cancel-1].handle == cancel_handle))
 					continue;
 				if (cancel_handle->arbiter != arbiter)
 					break;
 				_starpu_spin_lock(&cancel_handle->header_lock);
-				STARPU_ASSERT(cancel_handle->refcnt == 1);
+				STARPU_ASSERT(cancel_handle->refcnt >= 1);
 				cancel_handle->refcnt--;
 				STARPU_ASSERT(cancel_handle->busy_count > 0);
 				cancel_handle->busy_count--;
