@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2013, 2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2017  CNRS
- * Copyright (C) 2016  Inria
+ * Copyright (C) 2008-2014,2016-2018                      Université de Bordeaux
+ * Copyright (C) 2011,2012,2016                           Inria
+ * Copyright (C) 2010-2013,2015-2017,2019                 CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -163,7 +163,7 @@ void _starpu_tag_clear(void)
 	 * the global rwlock. This contradicts the lock order of
 	 * starpu_tag_wait_array. Should not be a problem in practice since
 	 * _starpu_tag_clear is called at shutdown only. */
-	struct _starpu_tag_table *entry, *tmp;
+	struct _starpu_tag_table *entry=NULL, *tmp=NULL;
 
 	HASH_ITER(hh, tag_htbl, entry, tmp)
 	{
@@ -246,7 +246,7 @@ static void _starpu_tag_add_succ(struct _starpu_tag *tag, struct _starpu_cg *cg)
 	if (tag->state == STARPU_DONE)
 	{
 		/* the tag was already completed sooner */
-		_starpu_notify_cg(cg);
+		_starpu_notify_cg(tag, cg);
 	}
 }
 
@@ -263,11 +263,16 @@ void _starpu_notify_tag_dependencies(struct _starpu_tag *tag)
 	tag->state = STARPU_DONE;
 	_STARPU_TRACE_TAG_DONE(tag);
 
-	_starpu_notify_cg_list(&tag->tag_successors);
+	_starpu_notify_cg_list(tag, &tag->tag_successors);
 
 	_starpu_spin_unlock(&tag->lock);
 }
 
+void _starpu_notify_job_start_tag_dependencies(struct _starpu_tag *tag, _starpu_notify_job_start_data *data)
+{
+	_starpu_notify_job_start_cg_list(tag, &tag->tag_successors, data);
+}
+
 void starpu_tag_restart(starpu_tag_t id)
 {
 	struct _starpu_tag *tag = gettag_struct(id);
@@ -463,3 +468,23 @@ int starpu_tag_wait(starpu_tag_t id)
 {
 	return starpu_tag_wait_array(1, &id);
 }
+
+struct starpu_task *starpu_tag_get_task(starpu_tag_t id)
+{
+	struct _starpu_tag_table *entry;
+	struct _starpu_tag *tag;
+
+	STARPU_PTHREAD_RWLOCK_WRLOCK(&tag_global_rwlock);
+	HASH_FIND_UINT64_T(tag_htbl, &id, entry);
+	STARPU_PTHREAD_RWLOCK_UNLOCK(&tag_global_rwlock);
+
+	if (!entry)
+		return NULL;
+	tag = entry->tag;
+
+	if (!tag->job)
+		return NULL;
+
+	return tag->job->task;
+}
+
