@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015,2017                                CNRS
- * Copyright (C) 2013                                     Inria
- * Copyright (C) 2013-2014,2016-2017                      Université de Bordeaux
+ * Copyright (C) 2013,2017                                Inria
+ * Copyright (C) 2017                                     CNRS
+ * Copyright (C) 2013,2014,2016,2017                      Université de Bordeaux
  * Copyright (C) 2013                                     Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@ extern "C"
 #endif
 
 #include <datawizard/copy_driver.h>
+#include <datawizard/malloc.h>
 
 /* interface to manipulate memory disk */
 void * _starpu_disk_alloc (unsigned node, size_t size) STARPU_ATTRIBUTE_MALLOC;
@@ -51,7 +52,7 @@ int starpu_disk_test_request(struct _starpu_async_channel *async_channel);
 void starpu_disk_free_request(struct _starpu_async_channel *async_channel);
 
 /* interface to compare memory disk */
-int _starpu_is_same_kind_disk(unsigned node1, unsigned node2);
+int _starpu_disk_can_copy(unsigned node1, unsigned node2);
 
 /* change disk flag */
 
