@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
  * Copyright (C) 2008-2015,2017                           Université de Bordeaux
- * Copyright (C) 2010-2011,2013,2015                      CNRS
+ * Copyright (C) 2012,2013,2016,2017                      Inria
+ * Copyright (C) 2010,2011,2013,2015,2017,2019            CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,6 +24,7 @@
 #endif
 
 #include <common/config.h>
+#include <common/list.h>
 
 #ifdef STARPU_USE_CUDA
 #include <cuda.h>
@@ -34,6 +35,10 @@
 #include <starpu_opencl.h>
 #endif
 
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+#include <mpi.h>
+#endif
+
 #ifdef __cplusplus
 extern "C"
 {
@@ -54,10 +59,31 @@ struct _starpu_mic_async_event
 };
 #endif
 
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+LIST_TYPE(_starpu_mpi_ms_event_request,
+        MPI_Request request;
+);
+
+struct _starpu_mpi_ms_async_event
+{
+        int is_sender;
+        struct _starpu_mpi_ms_event_request_list * requests;
+};
+#endif
+
+LIST_TYPE(_starpu_disk_backend_event,
+	void *backend_event;
+);
+
 struct _starpu_disk_async_event
 {
 	unsigned memory_node;
-	void *backend_event;
+        struct _starpu_disk_backend_event_list * requests;
+
+	void * ptr;
+	unsigned node;
+	size_t size;
+	starpu_data_handle_t handle;
 };
 
 /* this is a structure that can be queried to see whether an asynchronous
@@ -68,26 +94,34 @@ union _starpu_async_channel_event
 	struct
 	{
 		unsigned finished;
-		starpu_pthread_mutex_t mutex;
-		starpu_pthread_cond_t cond;
+		starpu_pthread_queue_t *queue;
 	};
 #endif
 #ifdef STARPU_USE_CUDA
-	cudaEvent_t cuda_event;
+        cudaEvent_t cuda_event;
 #endif
 #ifdef STARPU_USE_OPENCL
         cl_event opencl_event;
 #endif
-#ifdef STARPU_USE_MIC
-	struct _starpu_mic_async_event mic_event;
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+        struct _starpu_mpi_ms_async_event mpi_ms_event;
 #endif
-	struct _starpu_disk_async_event disk_event;
+#ifdef STARPU_USE_MIC
+        struct _starpu_mic_async_event mic_event;
+#endif
+        struct _starpu_disk_async_event disk_event;
 };
 
 struct _starpu_async_channel
 {
 	union _starpu_async_channel_event event;
-	enum starpu_node_kind type;
+	struct _starpu_node_ops *node_ops;
+        /* Which node to polling when needing ACK msg */
+        struct _starpu_mp_node *polling_node_sender;
+        struct _starpu_mp_node *polling_node_receiver;
+        /* Used to know if the acknowlegdment msg is arrived from sinks */
+        volatile int starpu_mp_common_finished_sender;
+        volatile int starpu_mp_common_finished_receiver;
 };
 
 void _starpu_wake_all_blocked_workers_on_node(unsigned nodeid);
