@@ -1,7 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2015, 2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2017  CNRS
+ * Copyright (C) 2008-2017                                Université de Bordeaux
+ * Copyright (C) 2011,2016,2017                           Inria
+ * Copyright (C) 2018                                     Federal University of Rio Grande do Sul (UFRGS)
+ * Copyright (C) 2010-2019                                CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,24 +24,16 @@
 #include <datawizard/datawizard.h>
 #include <datawizard/memory_nodes.h>
 #include <core/disk.h>
-
-/* TODO: This should be tuned according to driver capabilities
- * Data interfaces should also have to declare how many asynchronous requests
- * they have actually started (think of e.g. csr).
- */
-#define MAX_PENDING_REQUESTS_PER_NODE 20
-#define MAX_PENDING_PREFETCH_REQUESTS_PER_NODE 10
-#define MAX_PENDING_IDLE_REQUESTS_PER_NODE 1
-#define MAX_PUSH_TIME 1000 /* Maximum time in us that we can afford pushing requests before going back to the driver loop, e.g. for checking GPU task termination */
+#include <core/simgrid.h>
 
 /* requests that have not been treated at all */
-static struct _starpu_data_request_list data_requests[STARPU_MAXNODES];
-static struct _starpu_data_request_list prefetch_requests[STARPU_MAXNODES];
-static struct _starpu_data_request_list idle_requests[STARPU_MAXNODES];
+static struct _starpu_data_request_prio_list data_requests[STARPU_MAXNODES];
+static struct _starpu_data_request_prio_list prefetch_requests[STARPU_MAXNODES];
+static struct _starpu_data_request_prio_list idle_requests[STARPU_MAXNODES];
 static starpu_pthread_mutex_t data_requests_list_mutex[STARPU_MAXNODES];
 
 /* requests that are not terminated (eg. async transfers) */
-static struct _starpu_data_request_list data_requests_pending[STARPU_MAXNODES];
+static struct _starpu_data_request_prio_list data_requests_pending[STARPU_MAXNODES];
 static unsigned data_requests_npending[STARPU_MAXNODES];
 static starpu_pthread_mutex_t data_requests_pending_list_mutex[STARPU_MAXNODES];
 
@@ -47,20 +42,22 @@ void _starpu_init_data_request_lists(void)
 	unsigned i;
 	for (i = 0; i < STARPU_MAXNODES; i++)
 	{
-		_starpu_data_request_list_init(&data_requests[i]);
-		_starpu_data_request_list_init(&prefetch_requests[i]);
-		_starpu_data_request_list_init(&idle_requests[i]);
+		_starpu_data_request_prio_list_init(&data_requests[i]);
+		_starpu_data_request_prio_list_init(&prefetch_requests[i]);
+		_starpu_data_request_prio_list_init(&idle_requests[i]);
 
+#ifndef STARPU_DEBUG
 		/* Tell helgrind that we are fine with checking for list_empty
 		 * in _starpu_handle_node_data_requests, we will call it
 		 * periodically anyway */
-		STARPU_HG_DISABLE_CHECKING(data_requests[i]._head);
-		STARPU_HG_DISABLE_CHECKING(prefetch_requests[i]._head);
-		STARPU_HG_DISABLE_CHECKING(idle_requests[i]._head);
+		STARPU_HG_DISABLE_CHECKING(data_requests[i].tree.root);
+		STARPU_HG_DISABLE_CHECKING(prefetch_requests[i].tree.root);
+		STARPU_HG_DISABLE_CHECKING(idle_requests[i].tree.root);
+#endif
 
 		STARPU_PTHREAD_MUTEX_INIT(&data_requests_list_mutex[i], NULL);
 
-		_starpu_data_request_list_init(&data_requests_pending[i]);
+		_starpu_data_request_prio_list_init(&data_requests_pending[i]);
 		data_requests_npending[i] = 0;
 		STARPU_PTHREAD_MUTEX_INIT(&data_requests_pending_list_mutex[i], NULL);
 	}
@@ -72,7 +69,11 @@ void _starpu_deinit_data_request_lists(void)
 	unsigned i;
 	for (i = 0; i < STARPU_MAXNODES; i++)
 	{
+		_starpu_data_request_prio_list_deinit(&data_requests[i]);
+		_starpu_data_request_prio_list_deinit(&prefetch_requests[i]);
+		_starpu_data_request_prio_list_deinit(&idle_requests[i]);
 		STARPU_PTHREAD_MUTEX_DESTROY(&data_requests_pending_list_mutex[i]);
+		_starpu_data_request_prio_list_deinit(&data_requests_pending[i]);
 		STARPU_PTHREAD_MUTEX_DESTROY(&data_requests_list_mutex[i]);
 	}
 }
@@ -111,14 +112,6 @@ static void _starpu_data_request_unlink(struct _starpu_data_request *r)
 
 static void _starpu_data_request_destroy(struct _starpu_data_request *r)
 {
-	switch (r->async_channel.type)
-	{
-		case STARPU_DISK_RAM:
-			starpu_disk_free_request(&r->async_channel);
-			break;
-		default:
-			break;
-	}
 	//fprintf(stderr, "DESTROY REQ %p (%d) refcnt %d\n", r, node, r->refcnt);
 	_starpu_data_request_delete(r);
 }
@@ -131,7 +124,9 @@ struct _starpu_data_request *_starpu_create_data_request(starpu_data_handle_t ha
 							 enum starpu_data_access_mode mode,
 							 unsigned ndeps,
 							 unsigned is_prefetch,
-							 unsigned is_write_invalidation)
+							 int prio,
+							 unsigned is_write_invalidation,
+							 const char *origin)
 {
 	struct _starpu_data_request *r = _starpu_data_request_new();
 
@@ -139,17 +134,28 @@ struct _starpu_data_request *_starpu_create_data_request(starpu_data_handle_t ha
 
 	_starpu_spin_init(&r->lock);
 
+	_STARPU_TRACE_DATA_REQUEST_CREATED(handle, src_replicate?src_replicate->memory_node:-1, dst_replicate?dst_replicate->memory_node:-1, prio, is_prefetch);
+
+	r->origin = origin;
 	r->handle = handle;
 	r->src_replicate = src_replicate;
 	r->dst_replicate = dst_replicate;
 	r->mode = mode;
-	r->async_channel.type = STARPU_UNUSED;
+	r->async_channel.node_ops = NULL;
+        r->async_channel.starpu_mp_common_finished_sender = 0;
+        r->async_channel.starpu_mp_common_finished_receiver = 0;
+        r->async_channel.polling_node_sender = NULL;
+        r->async_channel.polling_node_receiver = NULL;
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+        r->async_channel.event.mpi_ms_event.requests = NULL;
+#endif
 	if (handling_node == -1)
 		handling_node = STARPU_MAIN_RAM;
 	r->handling_node = handling_node;
-	STARPU_ASSERT(handling_node == STARPU_MAIN_RAM || _starpu_memory_node_get_nworkers(handling_node));
+	STARPU_ASSERT(starpu_node_get_kind(handling_node) == STARPU_CPU_RAM || _starpu_memory_node_get_nworkers(handling_node));
 	r->completed = 0;
 	r->prefetch = is_prefetch;
+	r->prio = prio;
 	r->retval = -1;
 	r->ndeps = ndeps;
 	r->next_req_count = 0;
@@ -197,7 +203,17 @@ int _starpu_wait_data_request_completion(struct _starpu_data_request *r, unsigne
 	int do_delete = 0;
 	int completed;
 
-	unsigned local_node = _starpu_memory_node_get_local_key();
+#ifdef STARPU_SIMGRID
+	unsigned local_node = starpu_worker_get_local_memory_node();
+
+	starpu_pthread_wait_t wait;
+
+	starpu_pthread_wait_init(&wait);
+	/* We need to get woken both when requests finish on our node, and on
+	 * the target node of the request we are waiting for */
+	starpu_pthread_queue_register(&wait, &_starpu_simgrid_transfer_queue[local_node]);
+	starpu_pthread_queue_register(&wait, &_starpu_simgrid_transfer_queue[(unsigned) r->dst_replicate->memory_node]);
+#endif
 
 	struct _starpu_worker *worker = _starpu_get_local_worker_key();
 	enum _starpu_worker_status old_status = STATUS_UNKNOWN;
@@ -210,6 +226,10 @@ int _starpu_wait_data_request_completion(struct _starpu_data_request *r, unsigne
 
 	do
 	{
+#ifdef STARPU_SIMGRID
+		starpu_pthread_wait_reset(&wait);
+#endif
+
 		STARPU_SYNCHRONIZE();
 		if (STARPU_RUNNING_ON_VALGRIND)
 			completed = 1;
@@ -223,13 +243,18 @@ int _starpu_wait_data_request_completion(struct _starpu_data_request *r, unsigne
 			_starpu_spin_unlock(&r->lock);
 		}
 
+#ifndef STARPU_SIMGRID
 #ifndef STARPU_NON_BLOCKING_DRIVERS
 		/* XXX: shouldn't be needed, and doesn't work with chained requests anyway */
 		_starpu_wake_all_blocked_workers_on_node(r->handling_node);
+#endif
 #endif
 
-		_starpu_datawizard_progress(local_node, may_alloc);
+		_starpu_datawizard_progress(may_alloc);
 
+#ifdef STARPU_SIMGRID
+		starpu_pthread_wait_wait(&wait);
+#endif
 	}
 	while (1);
 
@@ -238,6 +263,12 @@ int _starpu_wait_data_request_completion(struct _starpu_data_request *r, unsigne
 		_starpu_set_worker_status(worker, old_status);
 	}
 
+#ifdef STARPU_SIMGRID
+	starpu_pthread_queue_unregister(&wait, &_starpu_simgrid_transfer_queue[local_node]);
+	starpu_pthread_queue_unregister(&wait, &_starpu_simgrid_transfer_queue[(unsigned) r->dst_replicate->memory_node]);
+	starpu_pthread_wait_destroy(&wait);
+#endif
+
 
 	retval = r->retval;
 	if (retval)
@@ -262,9 +293,7 @@ int _starpu_wait_data_request_completion(struct _starpu_data_request *r, unsigne
 void _starpu_post_data_request(struct _starpu_data_request *r)
 {
 	unsigned handling_node = r->handling_node;
-	/* We don't have a worker for disk nodes, these should have been posted to a main RAM node */
-	STARPU_ASSERT(starpu_node_get_kind(handling_node) != STARPU_DISK_RAM);
-	STARPU_ASSERT(handling_node == STARPU_MAIN_RAM || _starpu_memory_node_get_nworkers(handling_node));
+	STARPU_ASSERT(starpu_node_get_kind(handling_node) == STARPU_CPU_RAM || _starpu_memory_node_get_nworkers(handling_node));
 
 //	_STARPU_DEBUG("POST REQUEST\n");
 
@@ -281,11 +310,11 @@ void _starpu_post_data_request(struct _starpu_data_request *r)
 	/* insert the request in the proper list */
 	STARPU_PTHREAD_MUTEX_LOCK(&data_requests_list_mutex[handling_node]);
 	if (r->prefetch == 2)
-		_starpu_data_request_list_push_back(&idle_requests[handling_node], r);
+		_starpu_data_request_prio_list_push_back(&idle_requests[handling_node], r);
 	else if (r->prefetch)
-		_starpu_data_request_list_push_back(&prefetch_requests[handling_node], r);
+		_starpu_data_request_prio_list_push_back(&prefetch_requests[handling_node], r);
 	else
-		_starpu_data_request_list_push_back(&data_requests[handling_node], r);
+		_starpu_data_request_prio_list_push_back(&data_requests[handling_node], r);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_list_mutex[handling_node]);
 
 #ifndef STARPU_NON_BLOCKING_DRIVERS
@@ -374,6 +403,12 @@ static void starpu_handle_data_request_completion(struct _starpu_data_request *r
 
 	r->completed = 1;
 
+#ifdef STARPU_SIMGRID
+	/* Wake potential worker which was waiting for it */
+	if (dst_replicate)
+		_starpu_wake_all_blocked_workers_on_node(dst_replicate->memory_node);
+#endif
+
 	/* Remove a reference on the destination replicate for the request */
 	if (dst_replicate)
 	{
@@ -488,7 +523,7 @@ static int starpu_handle_data_request(struct _starpu_data_request *r, unsigned m
 		_starpu_spin_unlock(&handle->header_lock);
 
 		STARPU_PTHREAD_MUTEX_LOCK(&data_requests_pending_list_mutex[r->handling_node]);
-		_starpu_data_request_list_push_back(&data_requests_pending[r->handling_node], r);
+		_starpu_data_request_prio_list_push_back(&data_requests_pending[r->handling_node], r);
 		data_requests_npending[r->handling_node]++;
 		STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_pending_list_mutex[r->handling_node]);
 
@@ -502,10 +537,10 @@ static int starpu_handle_data_request(struct _starpu_data_request *r, unsigned m
 	return 0;
 }
 
-static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *reqlist, unsigned src_node, unsigned may_alloc, unsigned n, unsigned *pushed, unsigned prefetch)
+static int __starpu_handle_node_data_requests(struct _starpu_data_request_prio_list *reqlist, unsigned src_node, unsigned may_alloc, unsigned n, unsigned *pushed, unsigned prefetch)
 {
 	struct _starpu_data_request *r;
-	struct _starpu_data_request_list new_data_requests[prefetch + 1]; /* Indexed by prefetch level */
+	struct _starpu_data_request_prio_list new_data_requests[prefetch + 1]; /* Indexed by prefetch level */
 	unsigned i;
 	int ret = 0;
 
@@ -515,7 +550,7 @@ static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *
 	/* This is racy, but not posing problems actually, since we know we
 	 * will come back here to probe again regularly anyway.
 	 * Thus, do not expose this optimization to helgrind */
-	if (!STARPU_RUNNING_ON_VALGRIND && _starpu_data_request_list_empty(&reqlist[src_node]))
+	if (!STARPU_RUNNING_ON_VALGRIND && _starpu_data_request_prio_list_empty(&reqlist[src_node]))
 		return 0;
 #endif
 
@@ -532,7 +567,7 @@ static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *
 	STARPU_PTHREAD_MUTEX_LOCK(&data_requests_list_mutex[src_node]);
 #endif
 
-	if (_starpu_data_request_list_empty(&reqlist[src_node]))
+	if (_starpu_data_request_prio_list_empty(&reqlist[src_node]))
 	{
 		/* there is no request */
                 STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_list_mutex[src_node]);
@@ -542,17 +577,17 @@ static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *
 	/* There is an entry: we create a new empty list to replace the list of
 	 * requests, and we handle the request(s) one by one in the former
 	 * list, without concurrency issues.*/
-	struct _starpu_data_request_list local_list = reqlist[src_node];
-	_starpu_data_request_list_init(&reqlist[src_node]);
+	struct _starpu_data_request_prio_list local_list = reqlist[src_node];
+	_starpu_data_request_prio_list_init(&reqlist[src_node]);
 
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_list_mutex[src_node]);
 
 	for (i = 0; i <= prefetch; i++)
-		_starpu_data_request_list_init(&new_data_requests[i]);
+		_starpu_data_request_prio_list_init(&new_data_requests[i]);
 
 	double start = starpu_timing_now();
 	/* for all entries of the list */
-	while (!_starpu_data_request_list_empty(&local_list))
+	while (!_starpu_data_request_prio_list_empty(&local_list))
 	{
                 int res;
 
@@ -564,7 +599,7 @@ static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *
 			break;
 		}
 
-		r = _starpu_data_request_list_pop_front(&local_list);
+		r = _starpu_data_request_prio_list_pop_front_highest(&local_list);
 
 		res = starpu_handle_data_request(r, may_alloc, prefetch);
 		if (res != 0 && res != -EAGAIN)
@@ -572,7 +607,7 @@ static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *
 			/* handle is busy, or not enough memory, postpone for now */
 			ret = res;
 			/* Prefetch requests might have gotten promoted while in tmp list */
-			_starpu_data_request_list_push_back(&new_data_requests[r->prefetch], r);
+			_starpu_data_request_prio_list_push_back(&new_data_requests[r->prefetch], r);
 			if (prefetch)
 				/* Prefetching more there would make the situation even worse */
 				break;
@@ -588,29 +623,51 @@ static int __starpu_handle_node_data_requests(struct _starpu_data_request_list *
 	}
 
 	/* Push back requests we didn't handle on the proper list */
-	while (!_starpu_data_request_list_empty(&local_list))
+	while (!_starpu_data_request_prio_list_empty(&local_list))
 	{
-		r = _starpu_data_request_list_pop_front(&local_list);
+		r = _starpu_data_request_prio_list_pop_front_highest(&local_list);
 		/* Prefetch requests might have gotten promoted while in tmp list */
-		_starpu_data_request_list_push_back(&new_data_requests[r->prefetch], r);
+		_starpu_data_request_prio_list_push_back(&new_data_requests[r->prefetch], r);
 	}
+	_starpu_data_request_prio_list_deinit(&local_list);
 
 	for (i = 0; i <= prefetch; i++)
-		if (!_starpu_data_request_list_empty(&new_data_requests[i]))
+		if (!_starpu_data_request_prio_list_empty(&new_data_requests[i]))
 			break;
 
 	if (i <= prefetch)
 	{
 		STARPU_PTHREAD_MUTEX_LOCK(&data_requests_list_mutex[src_node]);
-		if (!(_starpu_data_request_list_empty(&new_data_requests[0])))
-			_starpu_data_request_list_push_list_front(&new_data_requests[0], &data_requests[src_node]);
-		if (prefetch >= 1 && !(_starpu_data_request_list_empty(&new_data_requests[1])))
-			_starpu_data_request_list_push_list_front(&new_data_requests[1], &prefetch_requests[src_node]);
-		if (prefetch >= 2 && !(_starpu_data_request_list_empty(&new_data_requests[2])))
-			_starpu_data_request_list_push_list_front(&new_data_requests[2], &idle_requests[src_node]);
+		if (!(_starpu_data_request_prio_list_empty(&new_data_requests[0])))
+		{
+			_starpu_data_request_prio_list_push_prio_list_back(&new_data_requests[0], &data_requests[src_node]);
+			data_requests[src_node] = new_data_requests[0];
+		}
+		if (prefetch >= 1 && !(_starpu_data_request_prio_list_empty(&new_data_requests[1])))
+		{
+			_starpu_data_request_prio_list_push_prio_list_back(&new_data_requests[1], &prefetch_requests[src_node]);
+			prefetch_requests[src_node] = new_data_requests[1];
+		}
+		if (prefetch >= 2 && !(_starpu_data_request_prio_list_empty(&new_data_requests[2])))
+		{
+			_starpu_data_request_prio_list_push_prio_list_back(&new_data_requests[2], &idle_requests[src_node]);
+			idle_requests[src_node] = new_data_requests[2];
+		}
 		STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_list_mutex[src_node]);
 
-#ifndef STARPU_NON_BLOCKING_DRIVERS
+#ifdef STARPU_SIMGRID
+		if (*pushed)
+		{
+			/* We couldn't process the request due to missing
+			 * space. Advance the clock a bit to let eviction have
+			 * the time to make some room for us. Ideally we should
+			 * rather have the caller block, and explicitly wait
+			 * for eviction to happen.
+			 */
+			MSG_process_sleep(0.000001);
+			_starpu_wake_all_blocked_workers_on_node(src_node);
+		}
+#elif !defined(STARPU_NON_BLOCKING_DRIVERS)
 		_starpu_wake_all_blocked_workers_on_node(src_node);
 #endif
 	}
@@ -637,14 +694,14 @@ static int _handle_pending_node_data_requests(unsigned src_node, unsigned force)
 {
 //	_STARPU_DEBUG("_starpu_handle_pending_node_data_requests ...\n");
 //
-	struct _starpu_data_request_list new_data_requests_pending;
+	struct _starpu_data_request_prio_list new_data_requests_pending;
 	unsigned taken, kept;
 
 #ifdef STARPU_NON_BLOCKING_DRIVERS
 	/* Here helgrind would should that this is an un protected access.
 	 * We however don't care about missing an entry, we will get called
 	 * again sooner or later. */
-	if (!STARPU_RUNNING_ON_VALGRIND && _starpu_data_request_list_empty(&data_requests_pending[src_node]))
+	if (!STARPU_RUNNING_ON_VALGRIND && _starpu_data_request_prio_list_empty(&data_requests_pending[src_node]))
 		return 0;
 #endif
 
@@ -662,26 +719,26 @@ static int _handle_pending_node_data_requests(unsigned src_node, unsigned force)
 		/* We really want to handle requests */
 		STARPU_PTHREAD_MUTEX_LOCK(&data_requests_pending_list_mutex[src_node]);
 
-	if (_starpu_data_request_list_empty(&data_requests_pending[src_node]))
+	if (_starpu_data_request_prio_list_empty(&data_requests_pending[src_node]))
 	{
 		/* there is no request */
 		STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_pending_list_mutex[src_node]);
 		return 0;
 	}
 	/* for all entries of the list */
-	struct _starpu_data_request_list local_list = data_requests_pending[src_node];
-	_starpu_data_request_list_init(&data_requests_pending[src_node]);
+	struct _starpu_data_request_prio_list local_list = data_requests_pending[src_node];
+	_starpu_data_request_prio_list_init(&data_requests_pending[src_node]);
 
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_pending_list_mutex[src_node]);
 
-	_starpu_data_request_list_init(&new_data_requests_pending);
+	_starpu_data_request_prio_list_init(&new_data_requests_pending);
 	taken = 0;
 	kept = 0;
 
-	while (!_starpu_data_request_list_empty(&local_list))
+	while (!_starpu_data_request_prio_list_empty(&local_list))
 	{
 		struct _starpu_data_request *r;
-		r = _starpu_data_request_list_pop_front(&local_list);
+		r = _starpu_data_request_prio_list_pop_front_highest(&local_list);
 		taken++;
 
 		starpu_data_handle_t handle = r->handle;
@@ -699,7 +756,7 @@ static int _handle_pending_node_data_requests(unsigned src_node, unsigned force)
 			if (_starpu_spin_trylock(&handle->header_lock))
 			{
 				/* Handle is busy, retry this later */
-				_starpu_data_request_list_push_back(&new_data_requests_pending, r);
+				_starpu_data_request_prio_list_push_back(&new_data_requests_pending, r);
 				kept++;
 				continue;
 			}
@@ -729,15 +786,16 @@ static int _handle_pending_node_data_requests(unsigned src_node, unsigned force)
 				_starpu_spin_unlock(&r->lock);
 				_starpu_spin_unlock(&handle->header_lock);
 
-				_starpu_data_request_list_push_back(&new_data_requests_pending, r);
+				_starpu_data_request_prio_list_push_back(&new_data_requests_pending, r);
 				kept++;
 			}
 		}
 	}
+	_starpu_data_request_prio_list_deinit(&local_list);
 	STARPU_PTHREAD_MUTEX_LOCK(&data_requests_pending_list_mutex[src_node]);
 	data_requests_npending[src_node] -= taken - kept;
 	if (kept)
-		_starpu_data_request_list_push_list_back(&data_requests_pending[src_node], &new_data_requests_pending);
+		_starpu_data_request_prio_list_push_prio_list_back(&data_requests_pending[src_node], &new_data_requests_pending);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_pending_list_mutex[src_node]);
 
 	return taken - kept;
@@ -761,9 +819,9 @@ int _starpu_check_that_no_data_request_exists(unsigned node)
 	int no_pending;
 
 	STARPU_PTHREAD_MUTEX_LOCK(&data_requests_list_mutex[node]);
-	no_request = _starpu_data_request_list_empty(&data_requests[node])
-	          && _starpu_data_request_list_empty(&prefetch_requests[node])
-		  && _starpu_data_request_list_empty(&idle_requests[node]);
+	no_request = _starpu_data_request_prio_list_empty(&data_requests[node])
+	          && _starpu_data_request_prio_list_empty(&prefetch_requests[node])
+		  && _starpu_data_request_prio_list_empty(&idle_requests[node]);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_list_mutex[node]);
 	STARPU_PTHREAD_MUTEX_LOCK(&data_requests_pending_list_mutex[node]);
 	no_pending = !data_requests_npending[node];
@@ -798,38 +856,21 @@ void _starpu_update_prefetch_status(struct _starpu_data_request *r, unsigned pre
 
 	/* The request can be in a different list (handling request or the temp list)
 	 * we have to check that it is really in the prefetch list. */
-	struct _starpu_data_request *r_iter;
-	for (r_iter = _starpu_data_request_list_begin(&prefetch_requests[r->handling_node]);
-	     r_iter != _starpu_data_request_list_end(&prefetch_requests[r->handling_node]);
-	     r_iter = _starpu_data_request_list_next(r_iter))
+	if (_starpu_data_request_prio_list_ismember(&prefetch_requests[r->handling_node], r))
 	{
-
-		if (r==r_iter)
-		{
-			_starpu_data_request_list_erase(&prefetch_requests[r->handling_node],r);
-			_starpu_data_request_list_push_back(&data_requests[r->handling_node],r);
-			goto found;
-		}
+		_starpu_data_request_prio_list_erase(&prefetch_requests[r->handling_node],r);
+		_starpu_data_request_prio_list_push_back(&data_requests[r->handling_node],r);
 	}
-
 	/* The request can be in a different list (handling request or the temp list)
 	 * we have to check that it is really in the idle list. */
-	for (r_iter = _starpu_data_request_list_begin(&idle_requests[r->handling_node]);
-	     r_iter != _starpu_data_request_list_end(&idle_requests[r->handling_node]);
-	     r_iter = _starpu_data_request_list_next(r_iter))
+	else if (_starpu_data_request_prio_list_ismember(&idle_requests[r->handling_node], r))
 	{
-
-		if (r==r_iter)
-		{
-			_starpu_data_request_list_erase(&idle_requests[r->handling_node],r);
-			if (prefetch == 1)
-				_starpu_data_request_list_push_back(&prefetch_requests[r->handling_node],r);
-			else
-				_starpu_data_request_list_push_back(&data_requests[r->handling_node],r);
-			goto found;
-		}
+		_starpu_data_request_prio_list_erase(&idle_requests[r->handling_node],r);
+		if (prefetch == 1)
+			_starpu_data_request_prio_list_push_back(&prefetch_requests[r->handling_node],r);
+		else
+			_starpu_data_request_prio_list_push_back(&data_requests[r->handling_node],r);
 	}
-found:
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data_requests_list_mutex[r->handling_node]);
 
 #ifndef STARPU_NON_BLOCKING_DRIVERS
