@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2010, 2012-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2013  CNRS
+ * Copyright (C) 2016,2017                                Inria
+ * Copyright (C) 2010,2011,2013,2015,2017                 CNRS
+ * Copyright (C) 2009-2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,13 +20,15 @@
 #include <common/config.h>
 #include <datawizard/datawizard.h>
 #include <datawizard/memalloc.h>
+#include <datawizard/memory_nodes.h>
 #include <core/workers.h>
 #include <core/progress_hook.h>
+#include <core/topology.h>
 #ifdef STARPU_SIMGRID
 #include <core/simgrid.h>
 #endif
 
-int __starpu_datawizard_progress(unsigned memory_node, unsigned may_alloc, unsigned push_requests)
+int ___starpu_datawizard_progress(unsigned memory_node, unsigned may_alloc, unsigned push_requests)
 {
 	int ret = 0;
 
@@ -64,7 +67,41 @@ int __starpu_datawizard_progress(unsigned memory_node, unsigned may_alloc, unsig
 	return ret;
 }
 
-void _starpu_datawizard_progress(unsigned memory_node, unsigned may_alloc)
+int __starpu_datawizard_progress(unsigned may_alloc, unsigned push_requests)
 {
-	__starpu_datawizard_progress(memory_node, may_alloc, 1);
+	struct _starpu_worker *worker = _starpu_get_local_worker_key();
+        unsigned memnode;
+
+	if (!worker)
+	{
+		/* Call from main application, only make RAM requests progress */
+		int ret = 0;
+		int nnumas = starpu_memory_nodes_get_numa_count();
+		int numa;
+		for (numa = 0; numa < nnumas; numa++)
+			ret |=  ___starpu_datawizard_progress(numa, may_alloc, push_requests);
+
+		return ret;
+	}
+	if (worker->set)
+		/* Runing one of the workers of a worker set. The reference for
+		 * driving memory is its worker 0 (see registrations in topology.c) */
+		worker = &worker->set->workers[0];
+
+	unsigned current_worker_id = worker->workerid;
+        int ret = 0;
+	unsigned nnodes = starpu_memory_nodes_get_count();
+
+        for (memnode = 0; memnode < nnodes; memnode++)
+        {
+                if (_starpu_worker_drives_memory[current_worker_id][memnode] == 1)
+                        ret |= ___starpu_datawizard_progress(memnode, may_alloc, push_requests);
+        }
+
+        return ret;
+}
+
+void _starpu_datawizard_progress(unsigned may_alloc)
+{
+        __starpu_datawizard_progress(may_alloc, 1);
 }
