@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2011,2014                           Université de Bordeaux
- * Copyright (C) 2010-2011,2013,2015                      CNRS
+ * Copyright (C) 2008-2011,2014,2019                      Université de Bordeaux
+ * Copyright (C) 2010,2011,2013,2015,2017                 CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,4 +30,7 @@ uint32_t _starpu_compute_buffers_footprint(struct starpu_perfmodel *model, struc
 /* Compute the footprint that characterizes the layout of the data handle. */
 uint32_t _starpu_compute_data_footprint(starpu_data_handle_t handle);
 
+/* Compute the footprint that characterizes the allocation of the data handle. */
+uint32_t _starpu_compute_data_alloc_footprint(starpu_data_handle_t handle);
+
 #endif // __FOOTPRINT_H__
