@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2012-2015,2017                           CNRS
- * Copyright (C) 2013,2015-2016, 2018                           Université de Bordeaux
+ * Copyright (C) 2011,2012                                Inria
+ * Copyright (C) 2011-2017,2019                           CNRS
+ * Copyright (C) 2011-2013,2015,2016,2018-2019            Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,18 +15,8 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
+
 #include <starpu.h>
-#include <common/config.h>
-#include <datawizard/coherency.h>
-#include <datawizard/copy_driver.h>
-#include <datawizard/filters.h>
-#include <datawizard/memory_nodes.h>
-#include <starpu_hash.h>
-#include <starpu_cuda.h>
-#include <starpu_opencl.h>
-#include <drivers/opencl/driver_opencl.h>
-#include <drivers/mic/driver_mic_source.h>
-#include <core/task.h>
 
 static int copy_ram_to_ram(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED, void *dst_interface, unsigned dst_node);
 #ifdef STARPU_USE_CUDA
@@ -84,7 +74,7 @@ static const struct starpu_data_copy_methods multiformat_copy_data_methods_s =
 
 static void register_multiformat_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
 static starpu_ssize_t allocate_multiformat_buffer_on_node(void *data_interface_, unsigned dst_node);
-static void *multiformat_handle_to_pointer(starpu_data_handle_t data_handle, unsigned node);
+static void *multiformat_to_pointer(void *data_interface, unsigned node);
 static int multiformat_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void free_multiformat_buffer_on_node(void *data_interface, unsigned node);
 static size_t multiformat_interface_get_size(starpu_data_handle_t handle);
@@ -106,7 +96,7 @@ struct starpu_data_interface_ops starpu_interface_multiformat_ops =
 {
 	.register_data_handle  = register_multiformat_handle,
 	.allocate_data_on_node = allocate_multiformat_buffer_on_node,
-	.handle_to_pointer     = multiformat_handle_to_pointer,
+	.to_pointer            = multiformat_to_pointer,
 	.pointer_is_inside     = multiformat_pointer_is_inside,
 	.free_data_on_node     = free_multiformat_buffer_on_node,
 	.copy_methods          = &multiformat_copy_data_methods_s,
@@ -120,11 +110,9 @@ struct starpu_data_interface_ops starpu_interface_multiformat_ops =
 	.get_mf_ops            = get_mf_ops
 };
 
-static void *multiformat_handle_to_pointer(starpu_data_handle_t handle, unsigned node)
+static void *multiformat_to_pointer(void *data_interface, unsigned node)
 {
-	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
-	struct starpu_multiformat_interface *multiformat_interface =
-		(struct starpu_multiformat_interface *) starpu_data_get_interface_on_node(handle, node);
+	struct starpu_multiformat_interface *multiformat_interface = data_interface;
 
 	switch(starpu_node_get_kind(node))
 	{
@@ -227,13 +215,6 @@ void starpu_multiformat_data_register(starpu_data_handle_t *handleptr,
 				      uint32_t nobjects,
 				      struct starpu_multiformat_data_interface_ops *format_ops)
 {
-	_starpu_codelet_check_deprecated_fields(format_ops->cpu_to_opencl_cl);
-	_starpu_codelet_check_deprecated_fields(format_ops->opencl_to_cpu_cl);
-	_starpu_codelet_check_deprecated_fields(format_ops->cpu_to_cuda_cl);
-	_starpu_codelet_check_deprecated_fields(format_ops->cuda_to_cpu_cl);
-	_starpu_codelet_check_deprecated_fields(format_ops->cpu_to_mic_cl);
-	_starpu_codelet_check_deprecated_fields(format_ops->mic_to_cpu_cl);
-
 	struct starpu_multiformat_interface multiformat =
 	{
 		.id         = STARPU_MULTIFORMAT_INTERFACE_ID,
@@ -432,16 +413,18 @@ static int copy_cuda_common(void *src_interface, unsigned src_node STARPU_ATTRIB
 					return -ENOMEM;
 			}
 			status = cudaMemcpy(dst_multiformat->cpu_ptr, src_multiformat->cpu_ptr, size, kind);
+			if (!status)
+				status = cudaThreadSynchronize();
 			if (STARPU_UNLIKELY(status))
-			{
 				STARPU_CUDA_REPORT_ERROR(status);
-			}
 			break;
 		}
 		case cudaMemcpyDeviceToHost:
 		{
 			size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
 			status = cudaMemcpy(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind);
+			if (!status)
+				status = cudaThreadSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 
@@ -451,6 +434,8 @@ static int copy_cuda_common(void *src_interface, unsigned src_node STARPU_ATTRIB
 		{
 			size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
 			status = cudaMemcpy(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind);
+			if (!status)
+				status = cudaThreadSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 			break;
@@ -508,6 +493,8 @@ static int copy_cuda_common_async(void *src_interface, unsigned src_node STARPU_
 		{
 			size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
 			status = cudaMemcpy(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind);
+			if (!status)
+				status = cudaThreadSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 
@@ -555,16 +542,17 @@ static int copy_cuda_peer_common(void *src_interface, unsigned src_node,
 
 	cudaError_t status;
 	int size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
-	int src_dev = _starpu_memory_node_get_devid(src_node);
-	int dst_dev = _starpu_memory_node_get_devid(dst_node);
+	int src_dev = starpu_memory_node_get_devid(src_node);
+	int dst_dev = starpu_memory_node_get_devid(dst_node);
 
 	if (stream)
 	{
-		_STARPU_TRACE_START_DRIVER_COPY_ASYNC(src_node, dst_node);
+		double start;
+		starpu_interface_start_driver_copy_async(src_node, dst_node, &start);
 		status = cudaMemcpyPeerAsync(dst_multiformat->cuda_ptr, dst_dev,
 					     src_multiformat->cuda_ptr, src_dev,
 					     size, stream);
-		_STARPU_TRACE_END_DRIVER_COPY_ASYNC(src_node, dst_node);
+		starpu_interface_end_driver_copy_async(src_node, dst_node, start);
 		/* All good ! Still, returning -EAGAIN, because we will need to
                    check the transfert completion later */
 		if (status == cudaSuccess)
@@ -576,10 +564,12 @@ static int copy_cuda_peer_common(void *src_interface, unsigned src_node,
 	status = cudaMemcpyPeer(dst_multiformat->cuda_ptr, dst_dev,
 				src_multiformat->cuda_ptr, src_dev,
 				size);
+	if (!status)
+		status = cudaThreadSynchronize();
 	if (STARPU_UNLIKELY(status != cudaSuccess))
 		STARPU_CUDA_REPORT_ERROR(status);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, size);
+	starpu_interface_data_copy(src_node, dst_node, size);
 
 	return 0;
 }
@@ -656,7 +646,7 @@ static int copy_ram_to_opencl_async(void *src_interface, unsigned src_node,
         if (STARPU_UNLIKELY(err))
                 STARPU_OPENCL_REPORT_ERROR(err);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, size);
+	starpu_interface_data_copy(src_node, dst_node, size);
 	return ret;
 }
 
@@ -682,7 +672,8 @@ static int copy_opencl_to_ram_async(void *src_interface, unsigned src_node,
 	if (dst_multiformat->opencl_ptr == NULL)
 	{
 		/* XXX : it is weird that we might have to allocate memory here... */
-		_STARPU_MALLOC(dst_multiformat->opencl_ptr, dst_multiformat->nx * dst_multiformat->ops->opencl_elemsize);
+		dst_multiformat->opencl_ptr = malloc(dst_multiformat->nx * dst_multiformat->ops->opencl_elemsize);
+		STARPU_ASSERT_MSG(dst_multiformat->opencl_ptr != NULL || dst_multiformat->nx * dst_multiformat->ops->opencl_elemsize == 0, "Cannot allocate %ld bytes\n", (long) (dst_multiformat->nx * dst_multiformat->ops->opencl_elemsize));
 	}
 	err = starpu_opencl_copy_opencl_to_ram((cl_mem)src_multiformat->opencl_ptr,
 					       src_node,
@@ -695,7 +686,7 @@ static int copy_opencl_to_ram_async(void *src_interface, unsigned src_node,
         if (STARPU_UNLIKELY(err))
                 STARPU_OPENCL_REPORT_ERROR(err);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, size);
+	starpu_interface_data_copy(src_node, dst_node, size);
 
 
 	return ret;
@@ -747,7 +738,7 @@ static int copy_mic_common_ram_to_mic(void *src_interface, unsigned src_node, vo
 
 	copy_func(src_multiformat->cpu_ptr, src_node, dst_multiformat->cpu_ptr, dst_node, size);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, size);
+	starpu_interface_data_copy(src_node, dst_node, size);
 
 	return 0;
 }
@@ -765,7 +756,7 @@ static int copy_mic_common_mic_to_ram(void *src_interface, unsigned src_node, vo
 	size_t size = src_multiformat->nx * src_multiformat->ops->mic_elemsize;
 	copy_func(src_multiformat->mic_ptr, src_node, dst_multiformat->mic_ptr, dst_node, size);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, size);
+	starpu_interface_data_copy(src_node, dst_node, size);
 
 	return 0;
 }
