@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016                           Inria
- * Copyright (C) 2009-2018                                Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2011-2014,2016,2017                      Inria
+ * Copyright (C) 2018                                     Federal University of Rio Grande do Sul (UFRGS)
+ * Copyright (C) 2010-2017, 2019                          CNRS
+ * Copyright (C) 2009-2019                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +29,7 @@
 #include <datawizard/memory_nodes.h>
 #include <datawizard/malloc.h>
 #include <core/simgrid.h>
+#include <core/task.h>
 
 #ifdef STARPU_SIMGRID
 #include <sys/mman.h>
@@ -35,6 +37,16 @@
 #include <smpi/smpi.h>
 #endif
 
+#ifdef STARPU_HAVE_HWLOC
+#include <hwloc.h>
+#ifndef HWLOC_API_VERSION
+#define HWLOC_OBJ_PU HWLOC_OBJ_PROC
+#endif
+#if HWLOC_API_VERSION < 0x00010b00
+#define HWLOC_OBJ_NUMANODE HWLOC_OBJ_NODE
+#endif
+#endif
+
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
@@ -56,6 +68,15 @@ static unsigned long _starpu_malloc_simulation_fold;
 #endif
 #endif
 
+static starpu_malloc_hook malloc_hook;
+static starpu_free_hook free_hook;
+
+void starpu_malloc_set_hooks(starpu_malloc_hook _malloc_hook, starpu_free_hook _free_hook)
+{
+	malloc_hook = _malloc_hook;
+	free_hook = _free_hook;
+}
+
 void starpu_malloc_set_align(size_t align)
 {
 	STARPU_ASSERT_MSG(!(align & (align - 1)), "Alignment given to starpu_malloc_set_align (%lu) must be a power of two", (unsigned long) align);
@@ -112,7 +133,28 @@ static struct starpu_codelet malloc_pinned_cl =
 };
 #endif
 
+/* Allocation in CPU RAM */
 int starpu_malloc_flags(void **A, size_t dim, int flags)
+{
+	return _starpu_malloc_flags_on_node(STARPU_MAIN_RAM, A, dim, flags);
+}
+
+/* Return whether we should pin the allocated data */
+static int _starpu_malloc_should_pin(int flags)
+{
+	if (flags & STARPU_MALLOC_PINNED && disable_pinning <= 0)
+	{
+		if (_starpu_can_submit_cuda_task())
+		{
+			return 1;
+		}
+//		if (_starpu_can_submit_opencl_task())
+//			return 1;
+	}
+	return 0;
+}
+
+int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int flags)
 {
 	int ret=0;
 
@@ -121,14 +163,14 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 	if (flags & STARPU_MALLOC_COUNT)
 	{
 		if (!(flags & STARPU_MALLOC_NORECLAIM))
-			while (starpu_memory_allocate(STARPU_MAIN_RAM, dim, flags) != 0)
+			while (starpu_memory_allocate(dst_node, dim, flags) != 0)
 			{
 				size_t freed;
 				size_t reclaim = 2 * dim;
 				_STARPU_DEBUG("There is not enough memory left, we are going to reclaim %ld\n", (long)reclaim);
-				_STARPU_TRACE_START_MEMRECLAIM(STARPU_MAIN_RAM,0);
-				freed = _starpu_memory_reclaim_generic(STARPU_MAIN_RAM, 0, reclaim);
-				_STARPU_TRACE_END_MEMRECLAIM(STARPU_MAIN_RAM,0);
+				_STARPU_TRACE_START_MEMRECLAIM(dst_node,0);
+				freed = _starpu_memory_reclaim_generic(dst_node, 0, reclaim);
+				_STARPU_TRACE_END_MEMRECLAIM(dst_node,0);
 				if (freed < dim && !(flags & STARPU_MEMORY_WAIT))
 				{
 					// We could not reclaim enough memory
@@ -137,24 +179,29 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 				}
 			}
 		else if (flags & STARPU_MEMORY_WAIT)
-			starpu_memory_allocate(STARPU_MAIN_RAM, dim, flags);
+			starpu_memory_allocate(dst_node, dim, flags);
 		else
-			starpu_memory_allocate(STARPU_MAIN_RAM, dim, flags | STARPU_MEMORY_OVERFLOW);
+			starpu_memory_allocate(dst_node, dim, flags | STARPU_MEMORY_OVERFLOW);
 	}
 
-	struct _starpu_machine_config *config = _starpu_get_machine_config();
-	if (flags & STARPU_MALLOC_PINNED && disable_pinning <= 0 && STARPU_RUNNING_ON_VALGRIND == 0 && config->conf.ncuda != 0)
+	if (malloc_hook)
 	{
+		ret = malloc_hook(dst_node, A, dim, flags);
+		goto end;
+	}
+
+	if (_starpu_malloc_should_pin(flags) && STARPU_RUNNING_ON_VALGRIND == 0)
+	{
+		if (_starpu_can_submit_cuda_task())
+		{
 #ifdef STARPU_SIMGRID
 		/* FIXME: CUDA seems to be taking 650µs every 1MiB.
 		 * Ideally we would simulate this batching in 1MiB requests
 		 * instead of computing an average value.
 		 */
-		if (_starpu_simgrid_cuda_malloc_cost())
-			MSG_process_sleep((float) dim * 0.000650 / 1048576.);
+			if (_starpu_simgrid_cuda_malloc_cost())
+				MSG_process_sleep((float) dim * 0.000650 / 1048576.);
 #else /* STARPU_SIMGRID */
-		if (_starpu_can_submit_cuda_task())
-		{
 #ifdef STARPU_USE_CUDA
 #ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 			cudaError_t cures;
@@ -195,7 +242,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 			goto end;
 #endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 #endif /* STARPU_USE_CUDA */
-		}
+//		}
 //		else if (_starpu_can_submit_opencl_task())
 //		{
 //#ifdef STARPU_USE_OPENCL
@@ -224,8 +271,8 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 //			STARPU_ASSERT(push_res != -ENODEV);
 //			goto end;
 //#endif /* STARPU_USE_OPENCL */
-//		}
 #endif /* STARPU_SIMGRID */
+		}
 	}
 
 #ifdef STARPU_SIMGRID
@@ -293,14 +340,24 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 		}
 #endif
 	}
-	else
 #endif
-	if (_starpu_can_submit_scc_task())
+#ifdef STARPU_HAVE_HWLOC
+	if (starpu_memory_nodes_get_numa_count() > 1)
 	{
-#ifdef STARPU_USE_SCC
-		_starpu_scc_allocate_shared_memory(A, dim);
+		struct _starpu_machine_config *config = _starpu_get_machine_config();
+		hwloc_topology_t hwtopology = config->topology.hwtopology;
+		hwloc_obj_t numa_node_obj = hwloc_get_obj_by_type(hwtopology, HWLOC_OBJ_NUMANODE, starpu_memory_nodes_numa_id_to_hwloclogid(dst_node));
+		hwloc_bitmap_t nodeset = numa_node_obj->nodeset;
+#if HWLOC_API_VERSION >= 0x00020000
+		*A = hwloc_alloc_membind(hwtopology, dim, nodeset, HWLOC_MEMBIND_BIND, HWLOC_MEMBIND_BYNODESET | HWLOC_MEMBIND_NOCPUBIND);
+#else
+		*A = hwloc_alloc_membind_nodeset(hwtopology, dim, nodeset, HWLOC_MEMBIND_BIND, HWLOC_MEMBIND_NOCPUBIND);
 #endif
+		//fprintf(stderr, "Allocation %lu bytes on NUMA node %d [%p]\n", (unsigned long) dim, starpu_memnode_get_numaphysid(dst_node), *A);
+		if (!*A)
+			ret = -ENOMEM;
 	}
+#endif /* STARPU_HAVE_HWLOC */
 	else
 #ifdef STARPU_HAVE_POSIX_MEMALIGN
 	if (_malloc_align != sizeof(void*))
@@ -327,16 +384,14 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 				ret = -ENOMEM;
 		}
 
-#if defined(STARPU_SIMGRID) || defined(STARPU_USE_CUDA)
 end:
-#endif
 	if (ret == 0)
 	{
 		STARPU_ASSERT_MSG(*A, "Failed to allocated memory of size %lu b\n", (unsigned long)dim);
 	}
 	else if (flags & STARPU_MALLOC_COUNT)
 	{
-		starpu_memory_deallocate(STARPU_MAIN_RAM, dim);
+		starpu_memory_deallocate(dst_node, dim);
 	}
 
 	return ret;
@@ -386,18 +441,31 @@ static struct starpu_codelet free_pinned_cl =
 
 int starpu_free_flags(void *A, size_t dim, int flags)
 {
-#ifndef STARPU_SIMGRID
-	if (flags & STARPU_MALLOC_PINNED && disable_pinning <= 0 && STARPU_RUNNING_ON_VALGRIND == 0)
+	return _starpu_free_flags_on_node(STARPU_MAIN_RAM, A, dim, flags);
+}
+
+int _starpu_free_flags_on_node(unsigned dst_node, void *A, size_t dim, int flags)
+{
+	if (free_hook)
+	{
+		free_hook(dst_node, A, dim, flags);
+		goto out;
+	}
+
+	if (_starpu_malloc_should_pin(flags) && STARPU_RUNNING_ON_VALGRIND == 0)
 	{
 		if (_starpu_can_submit_cuda_task())
 		{
+#ifdef STARPU_SIMGRID
+			/* TODO: simulate CUDA barrier */
+#else /* !STARPU_SIMGRID */
 #ifdef STARPU_USE_CUDA
 #ifndef STARPU_HAVE_CUDA_MEMCPY_PEER
-			if (!_starpu_is_initialized())
+			if (!starpu_is_initialized())
 			{
 #endif
-				/* This is especially useful when starpu_free is called from
-				 * the GCC-plugin. starpu_shutdown will probably have already
+				/* This is especially useful when starpu_free is called even
+				 * though starpu_shutdown has already
 				 * been called, so we will not be able to submit a task. */
 				cudaError_t err = cudaFreeHost(A);
 				if (STARPU_UNLIKELY(err))
@@ -428,6 +496,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 			}
 #endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 #endif /* STARPU_USE_CUDA */
+#endif /* STARPU_SIMGRID */
 		}
 //	else if (_starpu_can_submit_opencl_task())
 //	{
@@ -453,7 +522,6 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 //	}
 //#endif
 	}
-#endif /* STARPU_SIMGRID */
 
 #ifdef STARPU_SIMGRID
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
@@ -465,23 +533,22 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 		munmap(A, dim);
 #endif
 	}
-	else
 #endif
-	if (_starpu_can_submit_scc_task())
+#ifdef STARPU_HAVE_HWLOC
+	else if (starpu_memory_nodes_get_numa_count() > 1)
 	{
-#ifdef STARPU_USE_SCC
-		_starpu_scc_free_shared_memory(A);
-#endif
+		struct _starpu_machine_config *config = _starpu_get_machine_config();
+		hwloc_topology_t hwtopology = config->topology.hwtopology;
+		hwloc_free(hwtopology, A, dim);
 	}
+#endif /* STARPU_HAVE_HWLOC */
 	else
 		free(A);
 
-#if !defined(STARPU_SIMGRID) && defined(STARPU_USE_CUDA)
 out:
-#endif
 	if (flags & STARPU_MALLOC_COUNT)
 	{
-		starpu_memory_deallocate(STARPU_MAIN_RAM, dim);
+		starpu_memory_deallocate(dst_node, dim);
 	}
 
 	return 0;
@@ -492,20 +559,10 @@ int starpu_free(void *A)
 	return starpu_free_flags(A, 0, STARPU_MALLOC_PINNED);
 }
 
-#ifdef STARPU_SIMGRID
-static starpu_pthread_mutex_t cuda_alloc_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
-static starpu_pthread_mutex_t opencl_alloc_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
-#endif
-
-static uintptr_t
-_starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
+static uintptr_t _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 {
 	uintptr_t addr = 0;
 
-#if defined(STARPU_USE_CUDA) && !defined(STARPU_SIMGRID)
-	cudaError_t status;
-#endif
-
 	/* Handle count first */
 	if (flags & STARPU_MALLOC_COUNT)
 	{
@@ -515,124 +572,11 @@ _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 		flags &= ~STARPU_MALLOC_COUNT;
 	}
 
-	switch(starpu_node_get_kind(dst_node))
-	{
-		case STARPU_CPU_RAM:
-		{
-			starpu_malloc_flags((void**) &addr, size,
-#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
-					/* without memcpy_peer, we can not
-					 * allocated pinned memory, since it
-					 * requires waiting for a task, and we
-					 * may be called with a spinlock held
-					 */
-					flags & ~STARPU_MALLOC_PINNED
-#else
-					flags
-#endif
-					);
-			break;
-		}
-#if defined(STARPU_USE_CUDA) || defined(STARPU_SIMGRID)
-		case STARPU_CUDA_RAM:
-		{
-#ifdef STARPU_SIMGRID
-			static uintptr_t last[STARPU_MAXNODES];
-#ifdef STARPU_DEVEL
-#warning TODO: record used memory, using a simgrid property to know the available memory
-#endif
-			/* Sleep for the allocation */
-			STARPU_PTHREAD_MUTEX_LOCK(&cuda_alloc_mutex);
-			if (_starpu_simgrid_cuda_malloc_cost())
-				MSG_process_sleep(0.000175);
-			if (!last[dst_node])
-				last[dst_node] = 1<<10;
-			addr = last[dst_node];
-			last[dst_node]+=size;
-			STARPU_ASSERT(last[dst_node] >= addr);
-			STARPU_PTHREAD_MUTEX_UNLOCK(&cuda_alloc_mutex);
-#else
-			struct _starpu_worker *worker = _starpu_get_local_worker_key();
-			unsigned devid = _starpu_memory_node_get_devid(dst_node);
-			if (!worker || worker->arch != STARPU_CUDA_WORKER || worker->devid != devid)
-#if defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
-				starpu_cuda_set_device(devid);
-#else
-				STARPU_ASSERT_MSG(0, "CUDA peer access is not available with this version of CUDA");
-#endif
-			/* Check if there is free memory */
-			size_t cuda_mem_free, cuda_mem_total;
-			status = cudaMemGetInfo(&cuda_mem_free, &cuda_mem_total);
-			if (status == cudaSuccess &&
-					cuda_mem_free < (size*2))
-			{
-					addr = 0;
-					break;
-			}
-			status = cudaMalloc((void **)&addr, size);
-			if (!addr || (status != cudaSuccess))
-			{
-				if (STARPU_UNLIKELY(status != cudaErrorMemoryAllocation))
-					STARPU_CUDA_REPORT_ERROR(status);
-				addr = 0;
-			}
-#endif
-			break;
-		}
-#endif
-#if defined(STARPU_USE_OPENCL) || defined(STARPU_SIMGRID)
-	        case STARPU_OPENCL_RAM:
-		{
-#ifdef STARPU_SIMGRID
-				static uintptr_t last[STARPU_MAXNODES];
-				/* Sleep for the allocation */
-				STARPU_PTHREAD_MUTEX_LOCK(&opencl_alloc_mutex);
-				if (_starpu_simgrid_cuda_malloc_cost())
-					MSG_process_sleep(0.000175);
-				if (!last[dst_node])
-					last[dst_node] = 1<<10;
-				addr = last[dst_node];
-				last[dst_node]+=size;
-				STARPU_ASSERT(last[dst_node] >= addr);
-				STARPU_PTHREAD_MUTEX_UNLOCK(&opencl_alloc_mutex);
-#else
-                                int ret;
-				cl_mem ptr;
-
-				ret = starpu_opencl_allocate_memory(_starpu_memory_node_get_devid(dst_node), &ptr, size, CL_MEM_READ_WRITE);
-				if (ret)
-				{
-					addr = 0;
-				}
-				else
-				{
-					addr = (uintptr_t)ptr;
-				}
-#endif
-			break;
-		}
-#endif
-	        case STARPU_DISK_RAM:
-		{
-			addr = (uintptr_t) _starpu_disk_alloc(dst_node, size);
-			break;
-		}
-
-#ifdef STARPU_USE_MIC
-		case STARPU_MIC_RAM:
-			if (_starpu_mic_allocate_memory((void **)(&addr), size, dst_node))
-				addr = 0;
-			break;
-#endif
-#ifdef STARPU_USE_SCC
-		case STARPU_SCC_RAM:
-			if (_starpu_scc_allocate_memory((void **)(&addr), size, dst_node))
-				addr = 0;
-			break;
-#endif
-		default:
-			STARPU_ABORT();
-	}
+	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
+	if (node_ops && node_ops->malloc_on_node)
+		return node_ops->malloc_on_node(dst_node, size, flags);
+	else
+		STARPU_ABORT_MSG("No malloc_on_node function defined for node %s\n", _starpu_node_get_prefix(starpu_node_get_kind(dst_node)));
 
 	if (addr == 0)
 	{
@@ -643,99 +587,19 @@ _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 	return addr;
 }
 
-void
-_starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int flags)
+void _starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int flags)
 {
 	int count = flags & STARPU_MALLOC_COUNT;
 	flags &= ~STARPU_MALLOC_COUNT;
-	enum starpu_node_kind kind = starpu_node_get_kind(dst_node);
-	switch(kind)
-	{
-		case STARPU_CPU_RAM:
-			starpu_free_flags((void*)addr, size,
-#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
-					flags & ~STARPU_MALLOC_PINNED
-#else
-					flags
-#endif
-					);
-			break;
-#if defined(STARPU_USE_CUDA) || defined(STARPU_SIMGRID)
-		case STARPU_CUDA_RAM:
-		{
-#ifdef STARPU_SIMGRID
-			STARPU_PTHREAD_MUTEX_LOCK(&cuda_alloc_mutex);
-			/* Sleep for the free */
-			if (_starpu_simgrid_cuda_malloc_cost())
-				MSG_process_sleep(0.000750);
-			STARPU_PTHREAD_MUTEX_UNLOCK(&cuda_alloc_mutex);
-#else
-			cudaError_t err;
-			struct _starpu_worker *worker = _starpu_get_local_worker_key();
-			unsigned devid = _starpu_memory_node_get_devid(dst_node);
-			if (!worker || worker->arch != STARPU_CUDA_WORKER || worker->devid != devid)
-#if defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
-				starpu_cuda_set_device(devid);
-#else
-				STARPU_ASSERT_MSG(0, "CUDA peer access is not available with this version of CUDA");
-#endif
-			err = cudaFree((void*)addr);
-			if (STARPU_UNLIKELY(err != cudaSuccess
-#ifdef STARPU_OPENMP
-		/* When StarPU is used as Open Runtime support,
-		 * starpu_omp_shutdown() will usually be called from a
-		 * destructor, in which case cudaThreadExit() reports a
-		 * cudaErrorCudartUnloading here. There should not
-		 * be any remaining tasks running at this point so
-		 * we can probably ignore it without much consequences. */
-		&& err != cudaErrorCudartUnloading
-#endif /* STARPU_OPENMP */
-						))
-				STARPU_CUDA_REPORT_ERROR(err);
-#endif
-			break;
-		}
-#endif
-#if defined(STARPU_USE_OPENCL) || defined(STARPU_SIMGRID)
-                case STARPU_OPENCL_RAM:
-		{
-#ifdef STARPU_SIMGRID
-			STARPU_PTHREAD_MUTEX_LOCK(&opencl_alloc_mutex);
-			/* Sleep for the free */
-			if (_starpu_simgrid_cuda_malloc_cost())
-				MSG_process_sleep(0.000750);
-			STARPU_PTHREAD_MUTEX_UNLOCK(&opencl_alloc_mutex);
-#else
-			cl_int err;
-                        err = clReleaseMemObject((void*)addr);
-			if (STARPU_UNLIKELY(err != CL_SUCCESS))
-				STARPU_OPENCL_REPORT_ERROR(err);
-#endif
-                        break;
-		}
-#endif
-	        case STARPU_DISK_RAM:
-		{
-			_starpu_disk_free (dst_node, (void *) addr , size);
-			break;
-		}
 
-#ifdef STARPU_USE_MIC
-		case STARPU_MIC_RAM:
-			_starpu_mic_free_memory((void*) addr, size, dst_node);
-			break;
-#endif
-#ifdef STARPU_USE_SCC
-		case STARPU_SCC_RAM:
-			_starpu_scc_free_memory((void *) addr, dst_node);
-			break;
-#endif
-		default:
-			STARPU_ABORT();
-	}
+	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
+	if (node_ops && node_ops->free_on_node)
+		node_ops->free_on_node(dst_node, addr, size, flags);
+	else
+		STARPU_ABORT_MSG("No free_on_node function defined for node %s\n", _starpu_node_get_prefix(starpu_node_get_kind(dst_node)));
+
 	if (count)
 		starpu_memory_deallocate(dst_node, size);
-
 }
 
 int
@@ -883,11 +747,21 @@ static struct _starpu_chunk *_starpu_new_chunk(unsigned dst_node, int flags)
 	return chunk;
 }
 
+/* Return whether we should use our suballocator */
+static int _starpu_malloc_should_suballoc(unsigned dst_node, size_t size, int flags)
+{
+	return size <= CHUNK_ALLOC_MAX &&
+		(starpu_node_get_kind(dst_node) == STARPU_CUDA_RAM
+		 || (starpu_node_get_kind(dst_node) == STARPU_CPU_RAM
+		     && _starpu_malloc_should_pin(flags))
+		 );
+}
+
 uintptr_t
 starpu_malloc_on_node_flags(unsigned dst_node, size_t size, int flags)
 {
 	/* Big allocation, allocate normally */
-	if (size > CHUNK_ALLOC_MAX || starpu_node_get_kind(dst_node) != STARPU_CUDA_RAM)
+	if (!_starpu_malloc_should_suballoc(dst_node, size, flags))
 		return _starpu_malloc_on_node(dst_node, size, flags);
 
 	/* Round up allocation to block size */
@@ -985,7 +859,7 @@ void
 starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int flags)
 {
 	/* Big allocation, deallocate normally */
-	if (size > CHUNK_ALLOC_MAX || starpu_node_get_kind(dst_node) != STARPU_CUDA_RAM)
+	if (!_starpu_malloc_should_suballoc(dst_node, size, flags))
 	{
 		_starpu_free_on_node_flags(dst_node, addr, size, flags);
 		return;
