@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2017  CNRS
- * Copyright (C) 2011, 2012, 2016  INRIA
+ * Copyright (C) 2011-2013,2015-2017                      Inria
+ * Copyright (C) 2008-2019                                Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,11 +22,15 @@
 #include <limits.h>
 
 #include <core/workers.h>
-#include <sched_policies/fifo_queues.h>
+#include <sched_policies/prio_deque.h>
 #include <core/debug.h>
 #include <starpu_scheduler.h>
 #include <core/sched_policy.h>
 #include <core/debug.h>
+#include <core/task.h>
+
+/* Experimental (dead) code which needs to be tested, fixed... */
+/* #define USE_OVERLOAD */
 
 /*
  * Experimental code for improving data cache locality:
@@ -58,21 +62,24 @@
 #define MAX_LOCALITY 8
 
 /* Entry for queued_tasks_per_data: records that a queued task is accessing the data with locality flag */
-struct locality_entry {
+#ifdef USE_LOCALITY_TASKS
+struct locality_entry
+{
 	UT_hash_handle hh;
 	starpu_data_handle_t data;
 	struct starpu_task *task;
 };
+#endif
 
 struct _starpu_work_stealing_data_per_worker
 {
-	struct _starpu_fifo_taskq *queue_array;
+	struct _starpu_prio_deque queue;
+	int running;
 	int *proxlist;
-	starpu_pthread_mutex_t worker_mutex;
 	int busy;
 
 #ifdef USE_LOCALITY_TASKS
-	/* This records the same as queue_array, but hashed by data accessed with locality flag.  */
+	/* This records the same as queue, but hashed by data accessed with locality flag.  */
 	/* FIXME: we record only one task per data, assuming that the access is
 	 * RW, and thus only one task is ready to write to it. Do we really need to handle the R case too? */
 	struct locality_entry *queued_tasks_per_data;
@@ -85,12 +92,11 @@ struct _starpu_work_stealing_data_per_worker
 
 struct _starpu_work_stealing_data
 {
+	int (*select_victim)(struct _starpu_work_stealing_data *, unsigned, int);
 	struct _starpu_work_stealing_data_per_worker *per_worker;
-	unsigned rr_worker;
 	/* keep track of the work performed from the beginning of the algorithm to make
 	 * better decisions about which queue to select when stealing or deferring work
 	 */
-	unsigned performed_total;
 	unsigned last_pop_worker;
 	unsigned last_push_worker;
 };
@@ -112,9 +118,8 @@ static int calibration_value = 0;
  * the worker previously selected doesn't own any task,
  * then we return the first non-empty worker.
  */
-static int select_victim_round_robin(unsigned sched_ctx_id)
+static int select_victim_round_robin(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	unsigned worker = ws->last_pop_worker;
 	unsigned nworkers;
 	int *workerids = NULL;
@@ -128,9 +133,10 @@ static int select_victim_round_robin(unsigned sched_ctx_id)
 		/* Here helgrind would shout that this is unprotected, but we
 		 * are fine with getting outdated values, this is just an
 		 * estimation */
-		ntasks = ws->per_worker[workerids[worker]].queue_array->ntasks;
+		ntasks = ws->per_worker[workerids[worker]].queue.ntasks;
 
-		if (ntasks && ws->per_worker[workerids[worker]].busy)
+		if (ntasks && (ws->per_worker[workerids[worker]].busy
+					   || starpu_worker_is_blocked_in_parallel(workerids[worker])))
 			break;
 
 		worker = (worker + 1) % nworkers;
@@ -157,9 +163,8 @@ static int select_victim_round_robin(unsigned sched_ctx_id)
  * Return a worker to whom add a task.
  * Selecting a worker is done in a round-robin fashion.
  */
-static unsigned select_worker_round_robin(struct starpu_task *task, unsigned sched_ctx_id)
+static unsigned select_worker_round_robin(struct _starpu_work_stealing_data *ws, struct starpu_task *task, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	unsigned worker;
 	unsigned nworkers;
 	int *workerids;
@@ -168,7 +173,7 @@ static unsigned select_worker_round_robin(struct starpu_task *task, unsigned sch
 	worker = ws->last_push_worker;
 	do
 		worker = (worker + 1) % nworkers;
-	while (!starpu_worker_can_execute_task_first_impl(workerids[worker], task, NULL));
+	while (!ws->per_worker[workerids[worker]].running || !starpu_worker_can_execute_task_first_impl(workerids[worker], task, NULL));
 
 	ws->last_push_worker = worker;
 
@@ -177,9 +182,8 @@ static unsigned select_worker_round_robin(struct starpu_task *task, unsigned sch
 
 #ifdef USE_LOCALITY
 /* Select a worker according to the locality of the data of the task to be scheduled */
-static unsigned select_worker_locality(struct starpu_task *task, unsigned sched_ctx_id)
+static unsigned select_worker_locality(struct _starpu_work_stealing_data *ws, struct starpu_task *task, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
 	if (nbuffers == 0)
 		return -1;
@@ -187,10 +191,10 @@ static unsigned select_worker_locality(struct starpu_task *task, unsigned sched_
 	unsigned i, n;
 	unsigned ndata[STARPU_NMAXWORKERS] = { 0 };
 	int best_worker = -1;
-	unsigned best_ndata = 0;
 
 	n = 0;
 	for (i = 0; i < nbuffers; i++)
+	{
 		if (STARPU_TASK_GET_MODE(task, i) & STARPU_LOCALITY)
 		{
 			starpu_data_handle_t data = STARPU_TASK_GET_HANDLE(task, i);
@@ -199,19 +203,20 @@ static unsigned select_worker_locality(struct starpu_task *task, unsigned sched_
 				ndata[locality]++;
 			n++;
 		}
+	}
 
 	if (n)
 	{
 		/* Some locality buffers, choose worker which has most of them */
-
 		struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
 		struct starpu_sched_ctx_iterator it;
-		workers->init_iterator(workers, &it);
+		unsigned best_ndata = 0;
 
+		workers->init_iterator(workers, &it);
 		while(workers->has_next(workers, &it))
 		{
 			int workerid = workers->get_next(workers, &it);
-			if (ndata[workerid] > best_ndata && ws->per_worker[workerid].busy)
+			if (ndata[workerid] > best_ndata && ws->per_worker[workerid].running && ws->per_worker[workerid].busy)
 			{
 				best_worker = workerid;
 				best_ndata = ndata[workerid];
@@ -240,11 +245,10 @@ static void record_data_locality(struct starpu_task *task STARPU_ATTRIBUTE_UNUSE
 
 #ifdef USE_LOCALITY_TASKS
 /* Record in the worker which data it used last with the locality flag */
-static void record_worker_locality(struct starpu_task *task, int workerid, unsigned sched_ctx_id)
+static void record_worker_locality(struct _starpu_work_stealing_data *ws, struct starpu_task *task, int workerid, unsigned sched_ctx_id)
 {
 	/* Record where in locality data where the task went */
 	unsigned i;
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	struct _starpu_work_stealing_data_per_worker *data = &ws->per_worker[workerid];
 
 	data->nlast_locality = 0;
@@ -258,9 +262,8 @@ static void record_worker_locality(struct starpu_task *task, int workerid, unsig
 		}
 }
 /* Called when pushing a task to a queue */
-static void locality_pushed_task(struct starpu_task *task, int workerid, unsigned sched_ctx_id)
+static void locality_pushed_task(struct _starpu_work_stealing_data *ws, struct starpu_task *task, int workerid, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	struct _starpu_work_stealing_data_per_worker *data = &ws->per_worker[workerid];
 	unsigned i;
 	for (i = 0; i < STARPU_TASK_GET_NBUFFERS(task); i++)
@@ -285,7 +288,7 @@ static struct starpu_task *ws_pick_task(struct _starpu_work_stealing_data *ws, i
 	struct _starpu_work_stealing_data_per_worker *data_source = &ws->per_worker[source];
 	struct _starpu_work_stealing_data_per_worker *data_target = &ws->per_worker[target];
 	unsigned i, j, n = data_target->nlast_locality;
-	struct starpu_task *(tasks[MAX_LOCALITY]) = { NULL }, *best_task;
+	struct starpu_task *(tasks[MAX_LOCALITY]) = { NULL }, *best_task = NULL;
 	int ntasks[MAX_LOCALITY] = { 0 }, best_n; /* Number of locality data for this worker used by this task */
 	/* Look at the last data accessed by this worker */
 	STARPU_ASSERT(n < MAX_LOCALITY);
@@ -325,18 +328,21 @@ static struct starpu_task *ws_pick_task(struct _starpu_work_stealing_data *ws, i
 	if (best_n > 0)
 	{
 		/* found an interesting task, try to pick it! */
-		if (_starpu_fifo_pop_this_task(data_source->queue_array, target, best_task))
+		if (_starpu_prio_deque_pop_this_task(&data_source->queue, target, best_task))
 			return best_task;
 	}
 
 	/* Didn't find an interesting task, or couldn't run it :( */
-	return _starpu_fifo_pop_task(data_source->queue_array, target);
+	int skipped;
+	if (source != target)
+		return _starpu_prio_deque_deque_task_for_worker(&data_source->queue, target, &skipped);
+	else
+		return _starpu_prio_deque_pop_task_for_worker(&data_source->queue, target, &skipped);
 }
 
 /* Called when popping a task from a queue */
-static void locality_popped_task(struct starpu_task *task, int workerid, unsigned sched_ctx_id)
+static void locality_popped_task(struct _starpu_work_stealing_data *ws, struct starpu_task *task, int workerid, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	struct _starpu_work_stealing_data_per_worker *data = &ws->per_worker[workerid];
 	unsigned i;
 	for (i = 0; i < STARPU_TASK_GET_NBUFFERS(task); i++)
@@ -356,20 +362,24 @@ static void locality_popped_task(struct starpu_task *task, int workerid, unsigne
 		}
 }
 #else
-static void record_worker_locality(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
+static void record_worker_locality(struct _starpu_work_stealing_data *ws STARPU_ATTRIBUTE_UNUSED, struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
 {
 }
 /* Called when pushing a task to a queue */
-static void locality_pushed_task(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
+static void locality_pushed_task(struct _starpu_work_stealing_data *ws STARPU_ATTRIBUTE_UNUSED, struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
 {
 }
 /* Pick a task from workerid's queue, for execution on target */
 static struct starpu_task *ws_pick_task(struct _starpu_work_stealing_data *ws, int source, int target)
 {
-	return _starpu_fifo_pop_task(ws->per_worker[source].queue_array, target);
+	int skipped;
+	if (source != target)
+		return _starpu_prio_deque_deque_task_for_worker(&ws->per_worker[source].queue, target, &skipped);
+	else
+		return _starpu_prio_deque_pop_task_for_worker(&ws->per_worker[source].queue, target, &skipped);
 }
 /* Called when popping a task from a queue */
-static void locality_popped_task(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
+static void locality_popped_task(struct _starpu_work_stealing_data *ws STARPU_ATTRIBUTE_UNUSED, struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
 {
 }
 #endif
@@ -384,14 +394,13 @@ static void locality_popped_task(struct starpu_task *task STARPU_ATTRIBUTE_UNUSE
  * 		a smaller value implies a faster worker with an relatively emptier queue : more suitable to put tasks in
  * 		a bigger value implies a slower worker with an reletively more replete queue : more suitable to steal tasks from
  */
-static float overload_metric(unsigned sched_ctx_id, unsigned id)
+static float overload_metric(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id, unsigned id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	float execution_ratio = 0.0f;
 	float current_ratio = 0.0f;
 
-	int nprocessed = _starpu_get_deque_nprocessed(ws->per_worker[id].queue_array);
-	unsigned njobs = _starpu_get_deque_njobs(ws->per_worker[id].queue_array);
+	int nprocessed = _starpu_get_deque_nprocessed(ws->per_worker[id].queue);
+	unsigned njobs = _starpu_get_deque_njobs(ws->per_worker[id].queue);
 
 	/* Did we get enough information ? */
 	if (ws->performed_total > 0 && nprocessed > 0)
@@ -416,30 +425,26 @@ static float overload_metric(unsigned sched_ctx_id, unsigned id)
  * by the tasks are taken into account to select the most suitable
  * worker to steal task from.
  */
-static int select_victim_overload(unsigned sched_ctx_id)
+static int select_victim_overload(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-	unsigned worker;
-	float  worker_ratio;
 	unsigned best_worker = 0;
 	float best_ratio = FLT_MIN;
 
 	/* Don't try to play smart until we get
 	 * enough informations. */
 	if (ws->performed_total < calibration_value)
-		return select_victim_round_robin(sched_ctx_id);
+		return select_victim_round_robin(ws, sched_ctx_id);
 
 	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
-
 	struct starpu_sched_ctx_iterator it;
 
 	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
 	{
-                worker = workers->get_next(workers, &it);
-		worker_ratio = overload_metric(sched_ctx_id, worker);
+                unsigned worker = workers->get_next(workers, &it);
+		float worker_ratio = overload_metric(ws, sched_ctx_id, worker);
 
-		if (worker_ratio > best_ratio && ws->per_worker[worker].busy)
+		if (worker_ratio > best_ratio && ws->per_worker[worker].running && ws->per_worker[worker].busy)
 		{
 			best_worker = worker;
 			best_ratio = worker_ratio;
@@ -456,11 +461,9 @@ static int select_victim_overload(unsigned sched_ctx_id)
  * by the tasks are taken into account to select the most suitable
  * worker to add a task to.
  */
-static unsigned select_worker_overload(struct starpu_task *task, unsigned sched_ctx_id)
+static unsigned select_worker_overload(struct _starpu_work_stealing_data *ws, struct starpu_task *task, unsigned sched_ctx_id)
 {
 	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-	unsigned worker;
-	float  worker_ratio;
 	unsigned best_worker = 0;
 	float best_ratio = FLT_MAX;
 
@@ -470,18 +473,15 @@ static unsigned select_worker_overload(struct starpu_task *task, unsigned sched_
 		return select_worker_round_robin(task, sched_ctx_id);
 
 	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
-
 	struct starpu_sched_ctx_iterator it;
 
 	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
 	{
-		worker = workers->get_next(workers, &it);
+		unsigned worker = workers->get_next(workers, &it);
+		float worker_ratio = overload_metric(ws, sched_ctx_id, worker);
 
-		worker_ratio = overload_metric(sched_ctx_id, worker);
-
-		if (worker_ratio < best_ratio &&
-			starpu_worker_can_execute_task_first_impl(worker, task, NULL))
+		if (worker_ratio < best_ratio && ws->per_worker[worker].running && starpu_worker_can_execute_task_first_impl(worker, task, NULL))
 		{
 			best_worker = worker;
 			best_ratio = worker_ratio;
@@ -499,12 +499,13 @@ static unsigned select_worker_overload(struct starpu_task *task, unsigned sched_
  * This is a phony function used to call the right
  * function depending on the value of USE_OVERLOAD.
  */
-static inline int select_victim(unsigned sched_ctx_id)
+static inline int select_victim(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id,
+				     int workerid STARPU_ATTRIBUTE_UNUSED)
 {
 #ifdef USE_OVERLOAD
-	return select_victim_overload(sched_ctx_id);
+	return select_victim_overload(ws, sched_ctx_id);
 #else
-	return select_victim_round_robin(sched_ctx_id);
+	return select_victim_round_robin(ws, sched_ctx_id);
 #endif /* USE_OVERLOAD */
 }
 
@@ -513,12 +514,12 @@ static inline int select_victim(unsigned sched_ctx_id)
  * This is a phony function used to call the right
  * function depending on the value of USE_OVERLOAD.
  */
-static inline unsigned select_worker(struct starpu_task *task, unsigned sched_ctx_id)
+static inline unsigned select_worker(struct _starpu_work_stealing_data *ws, struct starpu_task *task, unsigned sched_ctx_id)
 {
 #ifdef USE_OVERLOAD
-	return select_worker_overload(task, sched_ctx_id);
+	return select_worker_overload(ws, task, sched_ctx_id);
 #else
-	return select_worker_round_robin(task, sched_ctx_id);
+	return select_worker_round_robin(ws, task, sched_ctx_id);
 #endif /* USE_OVERLOAD */
 }
 
@@ -534,78 +535,90 @@ static struct starpu_task *ws_pop_task(unsigned sched_ctx_id)
 	ws->per_worker[workerid].busy = 0;
 
 #ifdef STARPU_NON_BLOCKING_DRIVERS
-	if (STARPU_RUNNING_ON_VALGRIND || !_starpu_fifo_empty(ws->per_worker[workerid].queue_array))
+	if (STARPU_RUNNING_ON_VALGRIND || !_starpu_prio_deque_is_empty(&ws->per_worker[workerid].queue))
 #endif
 	{
-		STARPU_PTHREAD_MUTEX_LOCK(&ws->per_worker[workerid].worker_mutex);
 		task = ws_pick_task(ws, workerid, workerid);
 		if (task)
-			locality_popped_task(task, workerid, sched_ctx_id);
-		STARPU_PTHREAD_MUTEX_UNLOCK(&ws->per_worker[workerid].worker_mutex);
+			locality_popped_task(ws, task, workerid, sched_ctx_id);
 	}
 
-	if (task)
+	if(task)
 	{
 		/* there was a local task */
-		ws->performed_total++;
 		ws->per_worker[workerid].busy = 1;
+		if (_starpu_get_nsched_ctxs() > 1)
+		{
+			starpu_worker_relax_on();
+			_starpu_sched_ctx_lock_write(sched_ctx_id);
+			starpu_worker_relax_off();
+			starpu_sched_ctx_list_task_counters_decrement(sched_ctx_id, workerid);
+			if (_starpu_sched_ctx_worker_is_master_for_child_ctx(sched_ctx_id, workerid, task))
+				task = NULL;
+			_starpu_sched_ctx_unlock_write(sched_ctx_id);
+		}
 		return task;
 	}
 
-	starpu_pthread_mutex_t *sched_mutex;
-	starpu_pthread_cond_t *sched_cond;
-	starpu_worker_get_sched_condition(workerid, &sched_mutex, &sched_cond);
-	/* While stealing, relieve mutex used to synchronize with pushers */
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(sched_mutex);
-
-
 	/* we need to steal someone's job */
-	int victim = select_victim(sched_ctx_id);
+	starpu_worker_relax_on();
+	int victim = ws->select_victim(ws, sched_ctx_id, workerid);
+	starpu_worker_relax_off();
 	if (victim == -1)
 	{
-		STARPU_PTHREAD_MUTEX_LOCK_SCHED(sched_mutex);
 		return NULL;
 	}
 
-	if (STARPU_PTHREAD_MUTEX_TRYLOCK(&ws->per_worker[victim].worker_mutex))
+	if (_starpu_worker_trylock(victim))
 	{
 		/* victim is busy, don't bother it, come back later */
-		STARPU_PTHREAD_MUTEX_LOCK_SCHED(sched_mutex);
 		return NULL;
 	}
-	if (ws->per_worker[victim].queue_array != NULL && ws->per_worker[victim].queue_array->ntasks > 0)
+	if (ws->per_worker[victim].running && ws->per_worker[victim].queue.ntasks > 0)
 	{
 		task = ws_pick_task(ws, victim, workerid);
 	}
 
 	if (task)
 	{
-		ws->performed_total++;
-
 		_STARPU_TRACE_WORK_STEALING(workerid, victim);
-		_STARPU_TASK_BREAK_ON(task, sched);
+		starpu_sched_task_break(task);
+		starpu_sched_ctx_list_task_counters_decrement(sched_ctx_id, victim);
 		record_data_locality(task, workerid);
-		record_worker_locality(task, workerid, sched_ctx_id);
-		locality_popped_task(task, victim, sched_ctx_id);
+		record_worker_locality(ws, task, workerid, sched_ctx_id);
+		locality_popped_task(ws, task, victim, sched_ctx_id);
 	}
-	STARPU_PTHREAD_MUTEX_UNLOCK(&ws->per_worker[victim].worker_mutex);
-
-	/* Done with stealing, resynchronize with core */
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(sched_mutex);
+	starpu_worker_unlock(victim);
 
 #ifndef STARPU_NON_BLOCKING_DRIVERS
         /* While stealing, perhaps somebody actually give us a task, don't miss
          * the opportunity to take it before going to sleep. */
-	if (!task)
 	{
-		STARPU_PTHREAD_MUTEX_LOCK(&ws->per_worker[workerid].worker_mutex);
-		task = ws_pick_task(ws, workerid, workerid);
-		if (task)
-			locality_popped_task(task, workerid, sched_ctx_id);
-		STARPU_PTHREAD_MUTEX_UNLOCK(&ws->per_worker[workerid].worker_mutex);
+		struct _starpu_worker *worker = _starpu_get_worker_struct(starpu_worker_get_id());
+		if (!task && worker->state_keep_awake)
+		{
+			task = ws_pick_task(ws, workerid, workerid);
+			if (task)
+			{
+				/* keep_awake notice taken into account here, clear flag */
+				worker->state_keep_awake = 0;
+				locality_popped_task(ws, task, workerid, sched_ctx_id);
+			}
+		}
 	}
 #endif
 
+	if (task &&_starpu_get_nsched_ctxs() > 1)
+	{
+		starpu_worker_relax_on();
+		_starpu_sched_ctx_lock_write(sched_ctx_id);
+		starpu_worker_relax_off();
+		if (_starpu_sched_ctx_worker_is_master_for_child_ctx(sched_ctx_id, workerid, task))
+			task = NULL;
+		_starpu_sched_ctx_unlock_write(sched_ctx_id);
+		if (!task)
+			return NULL;
+	}
 	ws->per_worker[workerid].busy = !!task;
 	return task;
 }
@@ -615,26 +628,10 @@ int ws_push_task(struct starpu_task *task)
 {
 	unsigned sched_ctx_id = task->sched_ctx;
 	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-
 	int workerid;
 
-	unsigned worker = 0;
-	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
-	struct starpu_sched_ctx_iterator it;
-	
-	workers->init_iterator(workers, &it);
-	/* !! C'est ballot de tout locker! */
-	while(workers->has_next(workers, &it))
-	{
-		worker = workers->get_next(workers, &it);
-		starpu_pthread_mutex_t *sched_mutex;
-		starpu_pthread_cond_t *sched_cond;
-		starpu_worker_get_sched_condition(worker, &sched_mutex, &sched_cond);
-		STARPU_PTHREAD_MUTEX_LOCK_SCHED(sched_mutex);
-	}
-	
 #ifdef USE_LOCALITY
-	workerid = select_worker_locality(task, sched_ctx_id);
+	workerid = select_worker_locality(ws, task, sched_ctx_id);
 #else
 	workerid = -1;
 #endif
@@ -646,59 +643,46 @@ int ws_push_task(struct starpu_task *task)
 	 * context, we find the better one to put task on its queue */
 	if (workerid == -1 || !starpu_sched_ctx_contains_worker(workerid, sched_ctx_id) ||
 			!starpu_worker_can_execute_task_first_impl(workerid, task, NULL))
-		workerid = select_worker(task, sched_ctx_id);
-
-	record_data_locality(task, workerid);
-
+		workerid = select_worker(ws, task, sched_ctx_id);
+	starpu_worker_lock(workerid);
 	STARPU_AYU_ADDTOTASKQUEUE(starpu_task_get_job_id(task), workerid);
-	STARPU_PTHREAD_MUTEX_LOCK(&ws->per_worker[workerid].worker_mutex);
-	_STARPU_TASK_BREAK_ON(task, sched);
-	_starpu_fifo_push_task(ws->per_worker[workerid].queue_array, task);
-	locality_pushed_task(task, workerid, sched_ctx_id);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&ws->per_worker[workerid].worker_mutex);
+	starpu_sched_task_break(task);
+	record_data_locality(task, workerid);
+	STARPU_ASSERT_MSG(ws->per_worker[workerid].running, "workerid=%d, ws=%p\n", workerid, ws);
+	_starpu_prio_deque_push_back_task(&ws->per_worker[workerid].queue, task);
+	locality_pushed_task(ws, task, workerid, sched_ctx_id);
 
 	starpu_push_task_end(task);
+	starpu_worker_unlock(workerid);
+	starpu_sched_ctx_list_task_counters_increment(sched_ctx_id, workerid);
+
+#if !defined(STARPU_NON_BLOCKING_DRIVERS) || defined(STARPU_SIMGRID)
+	/* TODO: implement fine-grain signaling, similar to what eager does */
+	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
+	struct starpu_sched_ctx_iterator it;
 
 	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
-	{
-		worker = workers->get_next(workers, &it);
-		starpu_pthread_mutex_t *sched_mutex;
-		starpu_pthread_cond_t *sched_cond;
-		starpu_worker_get_sched_condition(worker, &sched_mutex, &sched_cond);
-#if !defined(STARPU_NON_BLOCKING_DRIVERS) || defined(STARPU_SIMGRID)
-		starpu_wakeup_worker_locked(worker, sched_cond, sched_mutex);
+		starpu_wake_worker_relax_light(workers->get_next(workers, &it));
 #endif
-		STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(sched_mutex);
-	}
-		
 	return 0;
 }
 
 static void ws_add_workers(unsigned sched_ctx_id, int *workerids,unsigned nworkers)
 {
 	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-
 	unsigned i;
-	int workerid;
 
 	for (i = 0; i < nworkers; i++)
 	{
-		workerid = workerids[i];
+		int workerid = workerids[i];
 		starpu_sched_ctx_worker_shares_tasks_lists(workerid, sched_ctx_id);
-		ws->per_worker[workerid].queue_array = _starpu_create_fifo();
+		_starpu_prio_deque_init(&ws->per_worker[workerid].queue);
+		ws->per_worker[workerid].running = 1;
 
-		/* Tell helgrid that we are fine with getting outdated values,
+		/* Tell helgrind that we are fine with getting outdated values,
 		 * this is just an estimation */
-		STARPU_HG_DISABLE_CHECKING(ws->per_worker[workerid].queue_array->ntasks);
-
-		/**
-		 * The first WS_POP_TASK will increase NPROCESSED though no task was actually performed yet,
-		 * we need to initialize it at -1.
-		 */
-		ws->per_worker[workerid].queue_array->nprocessed = -1;
-		ws->per_worker[workerid].queue_array->ntasks = 0;
-		STARPU_PTHREAD_MUTEX_INIT(&ws->per_worker[workerid].worker_mutex, NULL);
+		STARPU_HG_DISABLE_CHECKING(ws->per_worker[workerid].queue.ntasks);
 		ws->per_worker[workerid].busy = 0;
 		STARPU_HG_DISABLE_CHECKING(ws->per_worker[workerid].busy);
 	}
@@ -707,26 +691,21 @@ static void ws_add_workers(unsigned sched_ctx_id, int *workerids,unsigned nworke
 static void ws_remove_workers(unsigned sched_ctx_id, int *workerids, unsigned nworkers)
 {
 	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-
 	unsigned i;
-	int workerid;
 
 	for (i = 0; i < nworkers; i++)
 	{
-		workerid = workerids[i];
-		if (ws->per_worker[workerid].queue_array != NULL)
-		{
-			_starpu_destroy_fifo(ws->per_worker[workerid].queue_array);
-			ws->per_worker[workerid].queue_array = NULL;
-		}
-		STARPU_PTHREAD_MUTEX_DESTROY(&ws->per_worker[workerid].worker_mutex);
+		int workerid = workerids[i];
+
+		_starpu_prio_deque_destroy(&ws->per_worker[workerid].queue);
+		ws->per_worker[workerid].running = 0;
+		free(ws->per_worker[workerid].proxlist);
+		ws->per_worker[workerid].proxlist = NULL;
 	}
 }
 
 static void initialize_ws_policy(unsigned sched_ctx_id)
 {
-	starpu_sched_ctx_create_worker_collection(sched_ctx_id, STARPU_WORKER_LIST);
-
 	struct _starpu_work_stealing_data *ws;
 	_STARPU_MALLOC(ws, sizeof(struct _starpu_work_stealing_data));
 	starpu_sched_ctx_set_policy_data(sched_ctx_id, (void*)ws);
@@ -735,14 +714,16 @@ static void initialize_ws_policy(unsigned sched_ctx_id)
 	ws->last_push_worker = 0;
 	STARPU_HG_DISABLE_CHECKING(ws->last_pop_worker);
 	STARPU_HG_DISABLE_CHECKING(ws->last_push_worker);
+	ws->select_victim = select_victim;
 
-	/**
-	 * The first WS_POP_TASK will increase PERFORMED_TOTAL though no task was actually performed yet,
-	 * we need to initialize it at -1.
-	 */
-	ws->performed_total = -1;
+	unsigned nw = starpu_worker_get_count();
+	_STARPU_CALLOC(ws->per_worker, nw, sizeof(struct _starpu_work_stealing_data_per_worker));
 
-	_STARPU_CALLOC(ws->per_worker, STARPU_NMAXWORKERS, sizeof(struct _starpu_work_stealing_data_per_worker));
+	/* The application may use any integer */
+	if (starpu_sched_ctx_min_priority_is_set(sched_ctx_id) == 0)
+		starpu_sched_ctx_set_min_priority(sched_ctx_id, INT_MIN);
+	if (starpu_sched_ctx_max_priority_is_set(sched_ctx_id) == 0)
+		starpu_sched_ctx_set_max_priority(sched_ctx_id, INT_MAX);
 }
 
 static void deinit_ws_policy(unsigned sched_ctx_id)
@@ -750,8 +731,7 @@ static void deinit_ws_policy(unsigned sched_ctx_id)
 	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
 	free(ws->per_worker);
-        free(ws);
-        starpu_sched_ctx_delete_worker_collection(sched_ctx_id);
+	free(ws);
 }
 
 struct starpu_sched_policy _starpu_sched_ws_policy =
@@ -766,5 +746,120 @@ struct starpu_sched_policy _starpu_sched_ws_policy =
 	.post_exec_hook = NULL,
 	.pop_every_task = NULL,
 	.policy_name = "ws",
-	.policy_description = "work stealing"
+	.policy_description = "work stealing",
+	.worker_type = STARPU_WORKER_LIST,
+};
+
+/* local work stealing policy */
+/* Return a worker to steal a task from. The worker is selected according to
+ * the proximity list built using the info on te architecture provided by hwloc
+ */
+#ifdef STARPU_HAVE_HWLOC
+static int lws_select_victim(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id, int workerid)
+{
+	int nworkers = starpu_sched_ctx_get_nworkers(sched_ctx_id);
+	int i;
+	for (i = 0; i < nworkers; i++)
+	{
+		int neighbor = ws->per_worker[workerid].proxlist[i];
+                /* FIXME: do not keep looking again and again at some worker
+                 * which has tasks, but that can't execute on me */
+		int ntasks = ws->per_worker[neighbor].queue.ntasks;
+		if (ntasks && (ws->per_worker[neighbor].busy
+					   || starpu_worker_is_blocked_in_parallel(neighbor)))
+			return neighbor;
+	}
+	return -1;
+}
+#endif
+
+static void lws_add_workers(unsigned sched_ctx_id, int *workerids,
+			    unsigned nworkers)
+{
+	ws_add_workers(sched_ctx_id, workerids, nworkers);
+
+#ifdef STARPU_HAVE_HWLOC
+	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
+	/* Build a proximity list for every worker. It is cheaper to
+	 * build this once and then use it for popping tasks rather
+	 * than traversing the hwloc tree every time a task must be
+	 * stolen */
+	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
+	struct starpu_tree *tree = (struct starpu_tree*)workers->collection_private;
+	unsigned i;
+
+	/* get the complete list of workers (not just the added one) and rebuild the proxlists */
+	nworkers = starpu_sched_ctx_get_workers_list_raw(sched_ctx_id, &workerids);
+	for (i = 0; i < nworkers; i++)
+	{
+		int workerid = workerids[i];
+		if (ws->per_worker[workerid].proxlist == NULL)
+			_STARPU_MALLOC(ws->per_worker[workerid].proxlist, STARPU_NMAXWORKERS*sizeof(int));
+		int bindid;
+
+		struct starpu_sched_ctx_iterator it;
+		workers->init_iterator(workers, &it);
+
+		bindid   = starpu_worker_get_bindid(workerid);
+		it.value = starpu_tree_get(tree, bindid);
+		int cnt = 0;
+		for(;;)
+		{
+			struct starpu_tree *neighbour = (struct starpu_tree*)it.value;
+			int *neigh_workerids;
+			int neigh_nworkers = starpu_bindid_get_workerids(neighbour->id, &neigh_workerids);
+			int w;
+			for(w = 0; w < neigh_nworkers; w++)
+			{
+				if(!it.visited[neigh_workerids[w]] && workers->present[neigh_workerids[w]])
+				{
+					ws->per_worker[workerid].proxlist[cnt++] = neigh_workerids[w];
+					it.visited[neigh_workerids[w]] = 1;
+				}
+			}
+			if(!workers->has_next(workers, &it))
+				break;
+			it.value = it.possible_value;
+			it.possible_value = NULL;
+		}
+	}
+#endif
+}
+
+static void initialize_lws_policy(unsigned sched_ctx_id)
+{
+	/* lws is loosely based on ws, except that it might use hwloc. */
+	initialize_ws_policy(sched_ctx_id);
+
+	if (starpu_worker_get_count() != starpu_cpu_worker_get_count()
+			|| starpu_memory_nodes_get_numa_count() > 1
+		)
+	{
+		_STARPU_DISP("Warning: you are running the default lws scheduler, which is not a very smart scheduler, while the system has GPUs or several memory nodes. Make sure to read the StarPU documentation about adding performance models in order to be able to use the dmda or dmdas scheduler instead.\n");
+	}
+
+#ifdef STARPU_HAVE_HWLOC
+	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data *)starpu_sched_ctx_get_policy_data(sched_ctx_id);
+	ws->select_victim = lws_select_victim;
+#endif
+}
+
+struct starpu_sched_policy _starpu_sched_lws_policy =
+{
+	.init_sched = initialize_lws_policy,
+	.deinit_sched = deinit_ws_policy,
+	.add_workers = lws_add_workers,
+	.remove_workers = ws_remove_workers,
+	.push_task = ws_push_task,
+	.pop_task = ws_pop_task,
+	.pre_exec_hook = NULL,
+	.post_exec_hook = NULL,
+	.pop_every_task = NULL,
+	.policy_name = "lws",
+	.policy_description = "locality work stealing",
+#ifdef STARPU_HAVE_HWLOC
+	.worker_type = STARPU_WORKER_TREE,
+#else
+	.worker_type = STARPU_WORKER_LIST,
+#endif
 };
