@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Université de Bordeaux
+ * Copyright (C) 2011,2013,2014,2016                      Université de Bordeaux
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2011-2013,2017                           CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,11 +19,17 @@
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Test that when using starpu_data_acquire_cb, the callback is properly called
+ */
+
 unsigned token = 0;
 starpu_data_handle_t token_handle;
 
-void callback(void *arg STARPU_ATTRIBUTE_UNUSED)
+static
+void callback(void *arg)
 {
+	(void)arg;
 	token = 42;
         starpu_data_release(token_handle);
 }
@@ -30,11 +38,11 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-        ret = starpu_init(NULL);
+        ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	starpu_variable_data_register(&token_handle, 0, (uintptr_t)&token, sizeof(unsigned));
+	starpu_variable_data_register(&token_handle, STARPU_MAIN_RAM, (uintptr_t)&token, sizeof(unsigned));
         starpu_data_acquire_cb(token_handle, STARPU_RW, callback, NULL);
 
 	starpu_data_unregister(token_handle);
