@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
+ * Copyright (C) 2010,2011,2013,2014,2016,2017            Université de Bordeaux
  * Copyright (C) 2011-2013                                Inria
  * Copyright (C) 2010-2013,2015,2017                      CNRS
- * Copyright (C) 2010,2013-2014,2016-2017                 Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -30,14 +30,15 @@ static unsigned ntasks = 10000;
 #endif
 
 #ifdef STARPU_USE_CUDA
-extern void increment_cuda(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
+extern void increment_cuda(void *descr[], void *_args);
 #endif
 #ifdef STARPU_USE_OPENCL
 extern void increment_opencl(void *buffers[], void *args);
 #endif
 
-void increment_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void increment_cpu(void *descr[], void *arg)
 {
+	(void)arg;
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	(*tokenptr)++;
 }
@@ -74,8 +75,9 @@ int increment_token(void)
 }
 
 static
-void callback(void *arg STARPU_ATTRIBUTE_UNUSED)
+void callback(void *arg)
 {
+	(void)arg;
 	token++;
 	starpu_data_release(token_handle);
 }
