@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
  * Copyright (C) 2011-2016                                Université de Bordeaux
- * Copyright (C) 2011-2013,2015,2017                      CNRS
+ * Copyright (C) 2012,2013,2017                           Inria
+ * Copyright (C) 2011-2013,2017                           CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,8 @@ static starpu_data_handle_t handle;
 #ifdef STARPU_USE_CUDA
 static void redux_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -53,6 +55,8 @@ static void redux_cuda_kernel(void *descr[], void *arg)
 
 static void neutral_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -66,6 +70,8 @@ static void neutral_cuda_kernel(void *descr[], void *arg)
 #ifdef STARPU_USE_OPENCL
 static void redux_opencl_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned h_dst, h_src;
@@ -87,6 +93,8 @@ static void redux_opencl_kernel(void *descr[], void *arg)
 
 static void neutral_opencl_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned h_dst = 0;
@@ -101,6 +109,8 @@ static void neutral_opencl_kernel(void *descr[], void *arg)
 
 void redux_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -110,6 +120,8 @@ void redux_cpu_kernel(void *descr[], void *arg)
 
 void neutral_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -154,8 +166,9 @@ static struct starpu_codelet neutral_cl =
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+static void increment_opencl_kernel(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_mem d_token = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -174,6 +187,7 @@ static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE
 #ifdef STARPU_USE_CUDA
 static void increment_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -191,6 +205,7 @@ static void increment_cuda_kernel(void *descr[], void *arg)
 
 void increment_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
