@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2011-2013,2015,2017,2019                 CNRS
+ * Copyright (C) 2012,2013                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -13,7 +15,7 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
-#include <config.h>
+
 #include <starpu.h>
 #include "../test_interfaces.h"
 #include "../../../helper.h"
@@ -22,7 +24,7 @@
 #define HEIGHT 16
 
 #ifdef STARPU_USE_CPU
-static void test_matrix_cpu_func(void *buffers[], void *args);
+void test_matrix_cpu_func(void *buffers[], void *args);
 #endif /* !STARPU_USE_CPU */
 #ifdef STARPU_USE_CUDA
 extern void test_matrix_cuda_func(void *buffers[], void *_args);
@@ -45,10 +47,13 @@ struct test_config matrix_config =
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_matrix_opencl_func,
+#endif
+#ifdef STARPU_USE_MIC
+	.cpu_func_name = "test_matrix_cpu_func", 
 #endif
 	.handle        = &matrix_handle,
 	.dummy_handle  = &matrix2_handle,
-	.copy_failed   = 0,
+	.copy_failed   = SUCCESS,
 	.name          = "matrix_interface"
 };
 
@@ -64,14 +69,14 @@ register_data(void)
 		matrix[i] = i;
 
 	starpu_matrix_data_register(&matrix_handle,
-				    0,
+				    STARPU_MAIN_RAM,
 				    (uintptr_t) matrix,
 				    WIDTH, /* ld */
 				    WIDTH,
 				    HEIGHT,
 				    sizeof(matrix[0]));
 	starpu_matrix_data_register(&matrix2_handle,
-				    0,
+				    STARPU_MAIN_RAM,
 				    (uintptr_t) matrix2,
 				    WIDTH, /* ld */
 				    WIDTH,
@@ -86,7 +91,7 @@ unregister_data(void)
 	starpu_data_unregister(matrix2_handle);
 }
 
-static void
+void
 test_matrix_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
@@ -94,7 +99,7 @@ test_matrix_cpu_func(void *buffers[], void *args)
 	int *val;
 	int factor;
 	int i;
-	unsigned int nx, ny;
+	int nx, ny;
 
 	nx = STARPU_MATRIX_GET_NX(buffers[0]);
 	ny = STARPU_MATRIX_GET_NY(buffers[0]);
@@ -105,40 +110,36 @@ test_matrix_cpu_func(void *buffers[], void *args)
 	{
 		if (val[i] != i * factor)
 		{
-			matrix_config.copy_failed = 1;
+			matrix_config.copy_failed = FAILURE;
 			return;
 		}
 		val[i] *= -1;
 	}
 }
 
-int
-main(void)
+int main(int argc, char **argv)
 {
-	data_interface_test_summary *summary;
-	struct starpu_conf conf =
-	{
-		.ncpus   = -1,
-		.ncuda   = 2,
-		.nopencl = 1
-	};
+	struct data_interface_test_summary summary;
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	conf.ncuda = 2;
+	conf.nopencl = 1;
+	conf.nmic = -1;
 
-	if (starpu_init(&conf) == -ENODEV || starpu_cpu_worker_get_count() == 0)
+	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
 		goto enodev;
 
 	register_data();
 
-	summary = run_tests(&matrix_config);
-	if (!summary)
-		exit(EXIT_FAILURE);
+	run_tests(&matrix_config, &summary);
 
 	unregister_data();
 
 	starpu_shutdown();
 
-	data_interface_test_summary_print(stderr, summary);
+	data_interface_test_summary_print(stderr, &summary);
 
-	return data_interface_test_summary_success(summary);
+	return data_interface_test_summary_success(&summary);
 
 enodev:
 	return STARPU_TEST_SKIPPED;
