@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2013,2015-2016                      CNRS
- * Copyright (C) 2010,2013-2014,2016                      Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2010,2013,2014,2016                      Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,7 +24,7 @@
  */
 #define SIZE (100<<20)
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret;
 	char *var = NULL;
