@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2010-2011,2013-2014,2016,2018            Université de Bordeaux
  * Copyright (C) 2011-2013,2015,2017                      CNRS
+ * Copyright (C) 2010,2011,2013,2014,2016-2018            Université de Bordeaux
+ * Copyright (C) 2012,2017                                Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -32,13 +32,16 @@
 
 unsigned data, data2;
 
-void specific_kernel(STARPU_ATTRIBUTE_UNUSED void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void specific_kernel(void *descr[], void *arg)
 {
-	/* We do not protect this variable because it is only accessed when the
-	 * "data_handle" piece of data is accessed. */
+	(void)arg;
+	int node = starpu_task_get_current_data_node(0);
+	STARPU_ASSERT(node >= 0);
+	STARPU_ASSERT(starpu_node_get_kind(node) == STARPU_CPU_RAM);
 	unsigned *dataptr = (unsigned*) STARPU_VARIABLE_GET_PTR(descr[0]);
 
-	STARPU_ASSERT(dataptr == &data);
+	if (node == STARPU_MAIN_RAM)
+		STARPU_ASSERT(dataptr == &data);
 	(*dataptr)++;
 }
 
@@ -50,11 +53,18 @@ static struct starpu_codelet specific_cl =
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_RW},
 	.specific_nodes = 1,
-	.nodes = {STARPU_MAIN_RAM, -1},
+	.nodes = {STARPU_SPECIFIC_NODE_CPU, STARPU_SPECIFIC_NODE_LOCAL},
 };
 
+void cpu_codelet_unsigned_inc(void *descr[], void *arg)
+{
+	(void)arg;
+	unsigned *dataptr = (unsigned*) STARPU_VARIABLE_GET_PTR(descr[0]);
+	(*dataptr)++;
+}
+
 #ifdef STARPU_USE_CUDA
-void cuda_codelet_unsigned_inc(void *descr[], STARPU_ATTRIBUTE_UNUSED void *cl_arg);
+void cuda_codelet_unsigned_inc(void *descr[], void *cl_arg);
 #endif
 #ifdef STARPU_USE_OPENCL
 void opencl_codelet_unsigned_inc(void *buffers[], void *args);
@@ -62,7 +72,7 @@ void opencl_codelet_unsigned_inc(void *buffers[], void *args);
 
 static struct starpu_codelet cl =
 {
-	.cpu_funcs = {specific_kernel},
+	.cpu_funcs = {cpu_codelet_unsigned_inc},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {cuda_codelet_unsigned_inc},
 	.cuda_flags = {STARPU_CUDA_ASYNC},
@@ -79,7 +89,7 @@ static struct starpu_codelet cl =
 struct starpu_opencl_program opencl_program;
 #endif
 
-int main(STARPU_ATTRIBUTE_UNUSED int argc, STARPU_ATTRIBUTE_UNUSED char **argv)
+int main(void)
 {
 	starpu_data_handle_t data_handle, data_handle2;
 
@@ -126,6 +136,7 @@ int main(STARPU_ATTRIBUTE_UNUSED int argc, STARPU_ATTRIBUTE_UNUSED char **argv)
 	}
 
 	starpu_data_unregister(data_handle);
+	starpu_data_unregister(data_handle2);
 
 	ret = (data == ntasks) ? EXIT_SUCCESS : EXIT_FAILURE;
 
