@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012-2013  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2010-2014,2016                           Université de Bordeaux
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +24,10 @@
 #include "../helper.h"
 #include <common/thread.h>
 
+/*
+ * Mix submitting tasks and asynchronously acquiring the corresponding data.
+ */
+
 #define NBUFFERS_DEF	64
 #define NITER_DEF	128
 #define VECTORSIZE_DEF	1024
@@ -35,8 +40,10 @@ float *buffer[NBUFFERS_DEF];
 
 starpu_data_handle_t v_handle[NBUFFERS_DEF];
 
-static void dummy_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void dummy_codelet(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
 static struct starpu_codelet cl =
@@ -49,9 +56,11 @@ static struct starpu_codelet cl =
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {dummy_codelet},
 #endif
+	.cpu_funcs_name = {"dummy_codelet"},
 	.nbuffers = 1
 };
 
+static
 int use_handle(starpu_data_handle_t handle)
 {
 	int ret;
@@ -69,8 +78,10 @@ static starpu_pthread_mutex_t mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
 static starpu_pthread_cond_t cond = STARPU_PTHREAD_COND_INITIALIZER;
 static unsigned n_synced_buffers;
 
-void callback_sync_data(void *arg STARPU_ATTRIBUTE_UNUSED)
+static
+void callback_sync_data(void *arg)
 {
+	(void)arg;
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 
 	n_synced_buffers++;
@@ -91,7 +102,7 @@ int main(int argc, char **argv)
 	vectorsize /= 8;
 #endif
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -101,7 +112,7 @@ int main(int argc, char **argv)
 	{
 		ret = starpu_malloc((void **)&buffer[b], vectorsize);
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
-		starpu_vector_data_register(&v_handle[b], 0,
+		starpu_vector_data_register(&v_handle[b], STARPU_MAIN_RAM,
 				(uintptr_t)buffer[b], vectorsize, sizeof(char));
 		starpu_data_set_sequential_consistency_flag(v_handle[b], 0);
 	}
