@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2012-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2009-2016                                Université de Bordeaux
+ * Copyright (C) 2012,2016,2017                           Inria
+ * Copyright (C) 2010-2013,2015,2017,2019                 CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,7 +29,7 @@
 
 #if !defined(STARPU_HAVE_UNSETENV)
 #warning unsetenv is not defined. Skipping test
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -40,7 +41,6 @@ static void unset_env_variables(void)
 	(void) unsetenv("STARPU_NCUDA");
 	(void) unsetenv("STARPU_NOPENCL");
 	(void) unsetenv("STARPU_NMIC");
-	(void) unsetenv("STARPU_NSCC");
 }
 
 int main(int argc, char **argv)
@@ -56,23 +56,27 @@ int main(int argc, char **argv)
 	conf.ncuda = 0;
 	conf.nopencl = 0;
 	conf.nmic = 0;
-	conf.nscc = 0;
+        conf.nmpi_ms = 0;
 
 	/* starpu_init should return -ENODEV */
-	ret = starpu_initialize(&conf, &argc, &argv);
-	if (ret == -ENODEV)
-	     return EXIT_SUCCESS;
-	else
-	{
-	     	unsigned ncpu = starpu_cpu_worker_get_count();
-		unsigned ncuda = starpu_cuda_worker_get_count();
-		unsigned nopencl = starpu_opencl_worker_get_count();
-		FPRINTF(stderr, "StarPU has found :\n");
-		FPRINTF(stderr, "\t%u CPU cores\n", ncpu);
-		FPRINTF(stderr, "\t%u CUDA devices\n", ncuda);
-		FPRINTF(stderr, "\t%u OpenCL devices\n", nopencl);
-		return EXIT_FAILURE;
-	}
+        ret = starpu_initialize(&conf, &argc, &argv);
+        if (ret == -ENODEV)
+                return EXIT_SUCCESS;
+        else
+        {
+                unsigned ncpu = starpu_cpu_worker_get_count();
+                unsigned ncuda = starpu_cuda_worker_get_count();
+                unsigned nopencl = starpu_opencl_worker_get_count();
+                unsigned nmic = starpu_mic_worker_get_count();
+                unsigned nmpi_ms = starpu_mpi_ms_worker_get_count();
+                FPRINTF(stderr, "StarPU has found :\n");
+                FPRINTF(stderr, "\t%u CPU cores\n", ncpu);
+                FPRINTF(stderr, "\t%u CUDA devices\n", ncuda);
+                FPRINTF(stderr, "\t%u OpenCL devices\n", nopencl);
+                FPRINTF(stderr, "\t%u MIC devices\n", nmic);
+                FPRINTF(stderr, "\t%u MPI Master-Slaves devices\n", nmpi_ms);
+                return EXIT_FAILURE;
+        }
 
 
 }
