@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2012                                     Inria
- * Copyright (C) 2012-2013,2015,2017                      CNRS
- * Copyright (C) 2014,2016-2017                           Université de Bordeaux
+ * Copyright (C) 2012,2013,2015,2017                      CNRS
+ * Copyright (C) 2014,2016                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,7 @@
 
 #if !defined(STARPU_HAVE_UNSETENV)
 #warning unsetenv is not defined. Skipping test
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -88,13 +88,15 @@ static int test_cpu(void)
 	if (ret == -EINVAL)
 		return 1;
 
-
 	struct starpu_driver d =
 	{
 		.type = STARPU_CPU_WORKER,
 		.id.cpu_id = 0
 	};
 
+	conf.ncpus = 1;
+	conf.ncuda = 0;
+	conf.nopencl = 0;
 	conf.not_launched_drivers = &d;
 	conf.n_not_launched_drivers = 1;
 
@@ -145,13 +147,13 @@ static int test_cuda(void)
 	if (ret == -EINVAL)
 		return 1;
 
-
 	struct starpu_driver d =
 	{
 		.type = STARPU_CUDA_WORKER,
 		.id.cuda_id = 0
 	};
 
+	conf.ncpus = 0;
 	conf.ncuda = 1;
 	conf.nopencl = 0;
 	conf.not_launched_drivers = &d;
@@ -236,6 +238,7 @@ static int test_opencl(void)
 		.id.opencl_id = device_id
 	};
 
+	conf.ncpus = 0;
 	conf.ncuda = 0;
 	conf.nopencl = 1;
 	conf.not_launched_drivers = &d;
@@ -255,8 +258,6 @@ static int test_opencl(void)
 		return STARPU_TEST_SKIPPED;
 	}
 
-	starpu_topology_print(stdout);
-
 	init_driver(&d);
 	int i;
 	for (i = 0; i < NTASKS; i++)
@@ -281,8 +282,7 @@ static int test_opencl(void)
 }
 #endif /* STARPU_USE_OPENCL */
 
-int
-main(void)
+int main(void)
 {
 	int ret = STARPU_TEST_SKIPPED;
 
