@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012, 2014  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2010-2014,2016                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,14 +21,20 @@
 #include <unistd.h>
 #include "../helper.h"
 
+/*
+ * Test waiting for all tasks
+ */
+
 #ifdef STARPU_QUICK_CHECK
 static unsigned ntasks = 64;
 #else
 static unsigned ntasks = 65536;
 #endif
 
-static void dummy_func(void *descr[], void *arg)
+void dummy_func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet dummy_codelet =
@@ -35,6 +42,7 @@ static struct starpu_codelet dummy_codelet =
 	.cpu_funcs = {dummy_func},
 	.cuda_funcs = {dummy_func},
 	.opencl_funcs = {dummy_func},
+	.cpu_funcs_name = {"dummy_func"},
 	.model = NULL,
 	.nbuffers = 0
 };
@@ -89,7 +97,7 @@ int main(int argc, char **argv)
 
 	parse_args(argc, argv, &conf);
 
-	ret = starpu_init(&conf);
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
