@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2014  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010,2011,2013,2014,2016                 Université de Bordeaux
+ * Copyright (C) 2013                                     Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +24,10 @@
 #include <debug/starpu_debug_helpers.h>
 #include "../helper.h"
 
+/*
+ * Trigger a ping-pong test between two CUDA GPUs
+ */
+
 static size_t vector_size = 1;
 
 static int niter = 1000;
@@ -45,14 +50,14 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	/* Create a piece of data */
 	ret = starpu_malloc((void **)&v, vector_size);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
-	starpu_vector_data_register(&v_handle, 0, (uintptr_t)v, vector_size, 1);
+	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)v, vector_size, 1);
 
 	/* Find a pair of memory nodes */
 	if (starpu_cuda_worker_get_count() > 1)
