@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014  INRIA
+ * Copyright (C) 2014,2015,2017,2019                      CNRS
+ * Copyright (C) 2014,2016                                Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,8 +19,12 @@
 #include "../helper.h"
 #include <stdio.h>
 
+/*
+ * Check the starpu_omp_init/shutdown calls when called from constructor/destructor.
+ */
+
 #if !defined(STARPU_OPENMP)
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -28,6 +33,7 @@ __attribute__((constructor))
 static void omp_constructor(void)
 {
 	int ret = starpu_omp_init();
+	if (ret == -EINVAL) exit(STARPU_TEST_SKIPPED);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_omp_init");
 }
 
@@ -38,7 +44,7 @@ static void omp_destructor(void)
 }
 
 int
-main (int argc, char *argv[])
+main (void)
 {
 	return 0;
 }
