@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2011-2012,2014,2016                      Université de Bordeaux
- * Copyright (C) 2011-2015,2017                           CNRS
+ * Copyright (C) 2011,2012,2014                           Inria
+ * Copyright (C) 2011-2014,2017                           CNRS
+ * Copyright (C) 2011,2012,2014,2016                      Université de Bordeaux
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -42,7 +43,7 @@ static struct starpu_codelet cl =
 	.modes = {STARPU_W}
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	struct starpu_task task;
 	int ret;
