@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2013                                     Simon Archipoff
- * Copyright (C) 2013,2015-2017                           CNRS
- * Copyright (C) 2013-2016                                Université de Bordeaux
+ * Copyright (C) 2013,2015,2017                           CNRS
+ * Copyright (C) 2013-2016, 2019                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -70,6 +70,8 @@ run(struct starpu_sched_policy *policy)
 	{
 		.cpu_funcs = {A},
 		.cpu_funcs_name = {"A"},
+		.opencl_funcs = {A},
+		.cuda_funcs = {A},
 		.nbuffers = 0
 	};
 
@@ -77,6 +79,8 @@ run(struct starpu_sched_policy *policy)
 	{
 		.cpu_funcs = {B},
 		.cpu_funcs_name = {"B"},
+		.opencl_funcs = {B},
+		.cuda_funcs = {B},
 		.nbuffers = 0
 	};
 
@@ -113,21 +117,22 @@ enodev:
 	return -ENODEV;
 }
 
-int
-main(void)
+int main(void)
 {
 	struct starpu_sched_policy **policies;
 	struct starpu_sched_policy **policy;
 
-#ifdef STARPU_HAVE_UNSETENV
-	unsetenv("STARPU_SCHED");
-#endif
+	char *sched = getenv("STARPU_SCHED");
 
 	policies = starpu_sched_get_predefined_policies();
 	for(policy=policies ; *policy!=NULL ; policy++)
 	{
 		int ret;
 
+		if (sched && strcmp(sched, (*policy)->policy_name))
+			/* Testing another specific scheduler, no need to run this */
+			continue;
+
 		FPRINTF(stderr, "Running with policy %s.\n", (*policy)->policy_name);
 		ret = run(*policy);
 		if (ret == -ENODEV)
